from _typeshed import SupportsItems
from collections.abc import Callable, Iterable, Iterator, Sequence
from typing import Any, Literal, NamedTuple, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias

from webob._types import AsymmetricPropertyWithDelete

_T = TypeVar("_T")
_ListOrTuple: TypeAlias = list[_T] | tuple[_T, ...]
_ParsedAccept: TypeAlias = tuple[str, float, list[tuple[str, str]], list[str | tuple[str, str]]]

@type_check_only
class _SupportsStr(Protocol):
    def __str__(self) -> str: ...  # noqa: Y029

_AnyAcceptHeader: TypeAlias = AcceptValidHeader | AcceptInvalidHeader | AcceptNoHeader
_AnyAcceptCharsetHeader: TypeAlias = AcceptCharsetValidHeader | AcceptCharsetInvalidHeader | AcceptCharsetNoHeader
_AnyAcceptEncodingHeader: TypeAlias = AcceptEncodingValidHeader | AcceptEncodingInvalidHeader | AcceptEncodingNoHeader
_AnyAcceptLanguageHeader: TypeAlias = AcceptLanguageValidHeader | AcceptLanguageInvalidHeader | AcceptLanguageNoHeader

_AcceptProperty: TypeAlias = AsymmetricPropertyWithDelete[
    _AnyAcceptHeader,
    (
        _AnyAcceptHeader
        | SupportsItems[str, float | tuple[float, str]]
        | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
        | _SupportsStr
        | str
        | None
    ),
]
_AcceptCharsetProperty: TypeAlias = AsymmetricPropertyWithDelete[
    _AnyAcceptCharsetHeader,
    (
        _AnyAcceptCharsetHeader
        | SupportsItems[str, float]
        | _ListOrTuple[str | tuple[str, float] | list[Any]]
        | _SupportsStr
        | str
        | None
    ),
]
_AcceptEncodingProperty: TypeAlias = AsymmetricPropertyWithDelete[
    _AnyAcceptEncodingHeader,
    (
        _AnyAcceptEncodingHeader
        | SupportsItems[str, float]
        | _ListOrTuple[str | tuple[str, float] | list[Any]]
        | _SupportsStr
        | str
        | None
    ),
]
_AcceptLanguageProperty: TypeAlias = AsymmetricPropertyWithDelete[
    _AnyAcceptLanguageHeader,
    (
        _AnyAcceptLanguageHeader
        | SupportsItems[str, float]
        | _ListOrTuple[str | tuple[str, float] | list[Any]]
        | _SupportsStr
        | str
        | None
    ),
]

class AcceptOffer(NamedTuple):
    type: str
    subtype: str
    params: tuple[tuple[str, str], ...]

class Accept:
    @classmethod
    def parse(cls, value: str) -> Iterator[_ParsedAccept]: ...
    @classmethod
    def parse_offer(cls, offer: str | AcceptOffer) -> AcceptOffer: ...

class AcceptValidHeader(Accept):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[_ParsedAccept]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    def __add__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __radd__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def accept_html(self) -> bool: ...
    @property
    def accepts_html(self) -> bool: ...
    def acceptable_offers(self, offers: Sequence[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptInvalidOrNoHeader(Accept):
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    def accept_html(self) -> bool: ...
    @property
    def accepts_html(self) -> bool: ...
    def acceptable_offers(self, offers: Sequence[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float: ...

class AcceptNoHeader(_AcceptInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptValidHeader | Literal[""]) -> AcceptValidHeader: ...
    @overload
    def __add__(self, other: AcceptNoHeader | AcceptInvalidHeader | None) -> Self: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptValidHeader: ...
    @overload
    def __radd__(self, other: AcceptValidHeader | Literal[""]) -> AcceptValidHeader: ...
    @overload
    def __radd__(self, other: AcceptNoHeader | AcceptInvalidHeader | None) -> Self: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptValidHeader: ...

class AcceptInvalidHeader(_AcceptInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptValidHeader | Literal[""]) -> AcceptValidHeader: ...
    @overload
    def __add__(self, other: AcceptInvalidHeader | AcceptNoHeader | None) -> AcceptNoHeader: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __radd__(self, other: AcceptValidHeader | Literal[""]) -> AcceptValidHeader: ...
    @overload
    def __radd__(self, other: AcceptInvalidHeader | AcceptNoHeader | None) -> AcceptNoHeader: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptHeader
            | SupportsItems[str, float | tuple[float, str]]
            | _ListOrTuple[str | tuple[str, float, str] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptValidHeader | AcceptNoHeader: ...

@overload
def create_accept_header(header_value: AcceptValidHeader | Literal[""]) -> AcceptValidHeader: ...
@overload
def create_accept_header(header_value: AcceptInvalidHeader) -> AcceptInvalidHeader: ...
@overload
def create_accept_header(header_value: None | AcceptNoHeader) -> AcceptNoHeader: ...
@overload
def create_accept_header(header_value: str) -> AcceptValidHeader | AcceptInvalidHeader: ...
@overload
def create_accept_header(header_value: _AnyAcceptHeader | str | None) -> _AnyAcceptHeader: ...
def accept_property() -> _AcceptProperty: ...

class AcceptCharset:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptCharsetValidHeader(AcceptCharset):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    def __add__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __radd__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def acceptable_offers(self, offers: Sequence[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptCharsetInvalidOrNoHeader(AcceptCharset):
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptCharsetNoHeader(_AcceptCharsetInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __add__(self, other: AcceptCharsetInvalidHeader | AcceptCharsetNoHeader | Literal[""] | None) -> Self: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptCharsetValidHeader: ...
    @overload
    def __radd__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __radd__(self, other: AcceptCharsetInvalidHeader | AcceptCharsetNoHeader | Literal[""] | None) -> Self: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptCharsetValidHeader: ...

class AcceptCharsetInvalidHeader(_AcceptCharsetInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __add__(
        self, other: AcceptCharsetInvalidHeader | AcceptCharsetNoHeader | Literal[""] | None
    ) -> AcceptCharsetNoHeader: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __radd__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __radd__(
        self, other: AcceptCharsetInvalidHeader | AcceptCharsetNoHeader | Literal[""] | None
    ) -> AcceptCharsetNoHeader: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptCharsetHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...

@overload
def create_accept_charset_header(header_value: AcceptCharsetValidHeader | Literal[""]) -> AcceptCharsetValidHeader: ...
@overload
def create_accept_charset_header(header_value: AcceptCharsetInvalidHeader) -> AcceptCharsetInvalidHeader: ...
@overload
def create_accept_charset_header(header_value: AcceptCharsetNoHeader | None) -> AcceptCharsetNoHeader: ...
@overload
def create_accept_charset_header(header_value: str) -> AcceptCharsetValidHeader | AcceptCharsetInvalidHeader: ...
@overload
def create_accept_charset_header(header_value: _AnyAcceptCharsetHeader | str | None) -> _AnyAcceptCharsetHeader: ...
def accept_charset_property() -> _AcceptCharsetProperty: ...

class AcceptEncoding:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptEncodingValidHeader(AcceptEncoding):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    def __add__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __radd__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def acceptable_offers(self, offers: Sequence[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptEncodingInvalidOrNoHeader(AcceptEncoding):
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptEncodingNoHeader(_AcceptEncodingInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptEncodingValidHeader | Literal[""]) -> AcceptEncodingValidHeader: ...
    @overload
    def __add__(self, other: AcceptEncodingInvalidHeader | AcceptEncodingNoHeader | None) -> Self: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptEncodingValidHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingValidHeader | Literal[""]) -> AcceptEncodingValidHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingInvalidHeader | AcceptEncodingNoHeader | None) -> Self: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptEncodingValidHeader: ...

class AcceptEncodingInvalidHeader(_AcceptEncodingInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> None: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: AcceptEncodingValidHeader | Literal[""]) -> AcceptEncodingValidHeader: ...
    @overload
    def __add__(self, other: AcceptEncodingInvalidHeader | AcceptEncodingNoHeader | None) -> AcceptEncodingNoHeader: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingValidHeader | Literal[""]) -> AcceptEncodingValidHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingInvalidHeader | AcceptEncodingNoHeader | None) -> AcceptEncodingNoHeader: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptEncodingHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...

@overload
def create_accept_encoding_header(header_value: AcceptEncodingValidHeader | Literal[""]) -> AcceptEncodingValidHeader: ...
@overload
def create_accept_encoding_header(header_value: AcceptEncodingInvalidHeader) -> AcceptEncodingInvalidHeader: ...
@overload
def create_accept_encoding_header(header_value: AcceptEncodingNoHeader | None) -> AcceptEncodingNoHeader: ...
@overload
def create_accept_encoding_header(header_value: str) -> AcceptEncodingValidHeader | AcceptEncodingInvalidHeader: ...
@overload
def create_accept_encoding_header(header_value: _AnyAcceptEncodingHeader | str | None) -> _AnyAcceptEncodingHeader: ...
def accept_encoding_property() -> _AcceptEncodingProperty: ...

class AcceptLanguage:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptLanguageValidHeader(AcceptLanguage):
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __add__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __radd__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self: ...
    def basic_filtering(self, language_tags: Sequence[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    @overload
    def lookup(
        self, language_tags: Sequence[str], default_range: str | None, default_tag: str, default: None = None
    ) -> str | None: ...
    @overload
    def lookup(
        self, language_tags: Sequence[str], *, default_range: str | None = None, default_tag: str, default: None = None
    ) -> str | None: ...
    @overload
    def lookup(
        self, language_tags: Sequence[str], default_range: str | None, default_tag: None, default: _T | Callable[[], _T]
    ) -> _T | str | None: ...
    @overload
    def lookup(
        self, language_tags: Sequence[str], default_range: str | None, default_tag: str, default: _T | Callable[[], _T]
    ) -> _T | str: ...
    @overload
    def lookup(
        self,
        language_tags: Sequence[str],
        *,
        default_range: str | None = None,
        default_tag: None = None,
        default: _T | Callable[[], _T],
    ) -> _T | str | None: ...
    @overload
    def lookup(
        self, language_tags: Sequence[str], *, default_range: str | None = None, default_tag: str, default: _T | Callable[[], _T]
    ) -> _T | str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptLanguageInvalidOrNoHeader(AcceptLanguage):
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    def basic_filtering(self, language_tags: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    @overload
    def lookup(self, language_tags: object, default_range: object, default_tag: str, default: object = None) -> str: ...
    @overload
    def lookup(
        self, language_tags: object = None, *, default_range: object = None, default_tag: str, default: object = None
    ) -> str: ...
    @overload
    def lookup(self, language_tags: object, default_range: object, default_tag: None, default: _T | Callable[[], _T]) -> _T: ...
    @overload
    def lookup(
        self,
        language_tags: object = None,
        *,
        default_range: object = None,
        default_tag: None = None,
        default: _T | Callable[[], _T],
    ) -> _T: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptLanguageNoHeader(_AcceptLanguageInvalidOrNoHeader):
    def __init__(self) -> None: ...
    def copy(self) -> Self: ...
    @property
    def header_value(self) -> None: ...
    @property
    def parsed(self) -> None: ...
    @overload
    def __add__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __add__(self, other: AcceptLanguageInvalidHeader | AcceptLanguageNoHeader | Literal[""] | None) -> Self: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptLanguageValidHeader: ...
    @overload
    def __radd__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __radd__(self, other: AcceptLanguageInvalidHeader | AcceptLanguageNoHeader | Literal[""] | None) -> Self: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> Self | AcceptLanguageValidHeader: ...

class AcceptLanguageInvalidHeader(_AcceptLanguageInvalidOrNoHeader):
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> None: ...
    @overload
    def __add__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __add__(
        self, other: AcceptLanguageInvalidHeader | AcceptLanguageNoHeader | Literal[""] | None
    ) -> AcceptLanguageNoHeader: ...
    @overload
    def __add__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __radd__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __radd__(
        self, other: AcceptLanguageInvalidHeader | AcceptLanguageNoHeader | Literal[""] | None
    ) -> AcceptLanguageNoHeader: ...
    @overload
    def __radd__(
        self,
        other: (
            _AnyAcceptLanguageHeader
            | SupportsItems[str, float]
            | _ListOrTuple[str | tuple[str, float] | list[Any]]
            | _SupportsStr
            | str
            | None
        ),
    ) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...

@overload
def create_accept_language_header(header_value: AcceptLanguageValidHeader | Literal[""]) -> AcceptLanguageValidHeader: ...
@overload
def create_accept_language_header(header_value: AcceptLanguageNoHeader | None) -> AcceptLanguageNoHeader: ...
@overload
def create_accept_language_header(header_value: AcceptLanguageInvalidHeader) -> AcceptLanguageInvalidHeader: ...
@overload
def create_accept_language_header(header_value: str) -> AcceptLanguageValidHeader | AcceptLanguageInvalidHeader: ...
@overload
def create_accept_language_header(header_value: _AnyAcceptLanguageHeader | str | None) -> _AnyAcceptLanguageHeader: ...
def accept_language_property() -> _AcceptLanguageProperty: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["wiener_index", "schultz_index", "gutman_index"]

@_dispatchable
def wiener_index(G: Graph[_Node], weight: str | None = None) -> float: ...
@_dispatchable
def schultz_index(G: Graph[_Node], weight=None) -> float: ...
@_dispatchable
def gutman_index(G: Graph[_Node], weight=None) -> float: ...



from serial.tools.list_ports_common import ListPortInfo

class SysFS(ListPortInfo):
    usb_device_path: str | None
    device_path: str | None
    subsystem: str | None
    usb_interface_path: str | None
    def __init__(self, device: str) -> None: ...
    def read_line(self, *args: str) -> str | None: ...

def comports(include_links: bool = False) -> list[SysFS]: ...



import re
from collections.abc import Mapping
from typing import Any
from typing_extensions import deprecated

@deprecated("Use `list` instead.")
def bytes_to_intlist(bs: bytes) -> list[int]: ...
@deprecated("Use `bytes` instead.")
def intlist_to_bytes(xs: list[int]) -> bytes: ...
@deprecated("Use `yt_dlp.utils.jwt_encode` instead.")
def jwt_encode_hs256(payload_data: Any, key: str, headers: Mapping[str, Any] = {}) -> bytes: ...  # Passed to json.dumps().

compiled_regex_type: type[re.Pattern[Any]]



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import ClassVar

from authlib.jose.rfc7516 import JWEAlgorithmWithTagAwareKeyAgreement

class ECDH1PUAlgorithm(JWEAlgorithmWithTagAwareKeyAgreement):
    EXTRA_HEADERS: ClassVar[Iterable[str]]
    ALLOWED_KEY_CLS: Incomplete
    name: str
    description: str
    key_size: Incomplete
    aeskw: Incomplete
    def __init__(self, key_size=None) -> None: ...
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def compute_shared_key(self, shared_key_e, shared_key_s): ...
    def compute_fixed_info(self, headers, bit_size, tag): ...
    def compute_derived_key(self, shared_key, fixed_info, bit_size): ...
    def deliver_at_sender(self, sender_static_key, sender_ephemeral_key, recipient_pubkey, headers, bit_size, tag): ...
    def deliver_at_recipient(self, recipient_key, sender_static_pubkey, sender_ephemeral_pubkey, headers, bit_size, tag): ...
    def generate_keys_and_prepare_headers(self, enc_alg, key, sender_key, preset=None): ...
    def agree_upon_key_and_wrap_cek(self, enc_alg, headers, key, sender_key, epk, cek, tag): ...
    def wrap(self, enc_alg, headers, key, sender_key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key, sender_key, tag=None): ...

JWE_DRAFT_ALG_ALGORITHMS: Incomplete

def register_jwe_alg_draft(cls) -> None: ...



#!/usr/bin/env python3

"""
Check that the typeshed repository contains the correct files in the
correct places, and that various configuration files are correct.
"""

from __future__ import annotations

import os
import re
from pathlib import Path

from ts_utils.metadata import read_metadata
from ts_utils.paths import REQUIREMENTS_PATH, STDLIB_PATH, STUBS_PATH, TEST_CASES_DIR, TESTS_DIR, tests_path
from ts_utils.utils import (
    get_all_testcase_directories,
    get_gitignore_spec,
    parse_requirements,
    parse_stdlib_versions_file,
    spec_matches_path,
)

extension_descriptions = {".pyi": "stub", ".py": ".py"}

# These type checkers and linters must have exact versions in the requirements file to ensure
# consistent CI runs.
linters = {"mypy", "pyright", "ruff"}

ALLOWED_PY_FILES_IN_TESTS_DIR = {
    "django_settings.py"  # This file contains Django settings used by the mypy_django_plugin during stubtest execution.
}


def assert_consistent_filetypes(
    directory: Path, *, kind: str, allowed: set[str], allow_nonidentifier_filenames: bool = False
) -> None:
    """Check that given directory contains only valid Python files of a certain kind."""
    allowed_paths = {Path(f) for f in allowed}
    contents = list(directory.iterdir())
    gitignore_spec = get_gitignore_spec()
    while contents:
        entry = contents.pop()
        if spec_matches_path(gitignore_spec, entry):
            continue
        if entry.relative_to(directory) in allowed_paths:
            # Note if a subdirectory is allowed, we will not check its contents
            continue
        if entry.is_file():
            if not allow_nonidentifier_filenames:
                assert entry.stem.isidentifier(), f'Files must be valid modules, got: "{entry}"'
            bad_filetype = f'Only {extension_descriptions[kind]!r} files allowed in the "{directory}" directory; got: {entry}'
            assert entry.suffix == kind, bad_filetype
        else:
            assert entry.name.isidentifier(), f"Directories must be valid packages, got: {entry}"
            contents.extend(entry.iterdir())


def check_stdlib() -> None:
    """Check that the stdlib directory contains only the correct files."""
    assert_consistent_filetypes(STDLIB_PATH, kind=".pyi", allowed={"_typeshed/README.md", "VERSIONS", TESTS_DIR})
    check_tests_dir(tests_path("stdlib"))


def check_stubs() -> None:
    """Check that the stubs directory contains only the correct files."""
    gitignore_spec = get_gitignore_spec()
    for dist in STUBS_PATH.iterdir():
        if spec_matches_path(gitignore_spec, dist):
            continue
        assert dist.is_dir(), f"Only directories allowed in stubs, got {dist}"

        valid_dist_name = "^([A-Z0-9]|[A-Z0-9][A-Z0-9._-]*[A-Z0-9])$"  # courtesy of PEP 426
        assert re.fullmatch(
            valid_dist_name, dist.name, re.IGNORECASE
        ), f"Directory name must be a valid distribution name: {dist}"
        assert not dist.name.startswith("types-"), f"Directory name not allowed to start with 'types-': {dist}"

        allowed = {"METADATA.toml", "README", "README.md", "README.rst", TESTS_DIR}
        assert_consistent_filetypes(dist, kind=".pyi", allowed=allowed)

        tests_dir = tests_path(dist.name)
        if tests_dir.exists() and tests_dir.is_dir():
            check_tests_dir(tests_dir)


def check_tests_dir(tests_dir: Path) -> None:
    py_files_present = any(
        file.suffix == ".py" and file.name not in ALLOWED_PY_FILES_IN_TESTS_DIR for file in tests_dir.iterdir()
    )
    error_message = f"Test-case files must be in an `{TESTS_DIR}/{TEST_CASES_DIR}` directory, not in the `{TESTS_DIR}` directory"
    assert not py_files_present, error_message


def check_distutils() -> None:
    """Check whether all setuptools._distutils files are re-exported from distutils."""

    def all_relative_paths_in_directory(path: Path) -> set[Path]:
        return {pyi.relative_to(path) for pyi in path.rglob("*.pyi")}

    setuptools_path = STUBS_PATH / "setuptools" / "setuptools" / "_distutils"
    distutils_path = STUBS_PATH / "setuptools" / "distutils"
    all_setuptools_files = all_relative_paths_in_directory(setuptools_path)
    all_distutils_files = all_relative_paths_in_directory(distutils_path)
    assert all_setuptools_files and all_distutils_files, "Looks like this test might be out of date!"
    extra_files = all_setuptools_files - all_distutils_files
    joined = "\n".join(f"  * {distutils_path / f}" for f in extra_files)
    assert not extra_files, f"Files missing from distutils:\n{joined}"


def check_test_cases() -> None:
    """Check that the test_cases directory contains only the correct files."""
    for _, testcase_dir in get_all_testcase_directories():
        assert_consistent_filetypes(testcase_dir, kind=".py", allowed={"README.md"}, allow_nonidentifier_filenames=True)
        bad_test_case_filename = f'Files in a `{TEST_CASES_DIR}` directory must have names starting with "check_"; got "{{}}"'
        for file in testcase_dir.rglob("*.py"):
            assert file.stem.startswith("check_"), bad_test_case_filename.format(file)


def check_no_symlinks() -> None:
    """Check that there are no symlinks in the typeshed repository."""
    files = [Path(root, file) for root, _, files in os.walk(".") for file in files]
    no_symlink = "You cannot use symlinks in typeshed, please copy {} to its link."
    for file in files:
        if file.suffix == ".pyi" and file.is_symlink():
            raise ValueError(no_symlink.format(file))


def check_versions_file() -> None:
    """Check that the stdlib/VERSIONS file has the correct format."""
    version_map = parse_stdlib_versions_file()
    versions = list(version_map.keys())

    sorted_versions = sorted(versions)
    assert versions == sorted_versions, f"{versions=}\n\n{sorted_versions=}"

    modules = _find_stdlib_modules()
    # Sub-modules don't need to be listed in VERSIONS.
    extra = {m.split(".")[0] for m in modules} - version_map.keys()
    assert not extra, f"Modules not in versions: {extra}"
    extra = version_map.keys() - modules
    assert not extra, f"Versions not in modules: {extra}"


def _find_stdlib_modules() -> set[str]:
    modules = set[str]()
    for path, _, files in os.walk(STDLIB_PATH):
        for filename in files:
            base_module = ".".join(Path(path).parts[1:])
            if filename == "__init__.pyi":
                modules.add(base_module)
            elif filename.endswith(".pyi"):
                mod = filename[:-4]
                modules.add(f"{base_module}.{mod}" if base_module else mod)
    return modules


def check_metadata() -> None:
    """Check that all METADATA.toml files are valid."""
    for distribution in os.listdir(STUBS_PATH):
        # This function does various sanity checks for METADATA.toml files
        read_metadata(distribution)


def check_requirement_pins() -> None:
    """Check that type checkers and linters are pinned to an exact version."""
    requirements = parse_requirements()
    for package in linters:
        assert package in requirements, f"type checker/linter '{package}' not found in {REQUIREMENTS_PATH.name}"
        spec = requirements[package].specifier
        assert len(spec) == 1, f"type checker/linter '{package}' has complex specifier in {REQUIREMENTS_PATH.name}"
        msg = f"type checker/linter '{package}' is not pinned to an exact version in {REQUIREMENTS_PATH.name}"
        assert str(spec).startswith("=="), msg


if __name__ == "__main__":
    check_versions_file()
    check_metadata()
    check_requirement_pins()
    check_no_symlinks()
    check_stdlib()
    check_stubs()
    check_distutils()
    check_test_cases()



import optparse
from collections.abc import Callable, Collection, Iterator, Mapping
from typing import Any, Literal, NamedTuple, Protocol, TypedDict, type_check_only
from typing_extensions import NotRequired

from .extractor import gen_extractors, list_extractors
from .networking.impersonate import ImpersonateTarget
from .YoutubeDL import YoutubeDL

__all__ = ["YoutubeDL", "gen_extractors", "list_extractors", "main", "parse_options"]

@type_check_only
class _LoggerProtocol(Protocol):  # noqa: Y046
    def __init__(self, ydl: YoutubeDL | None = None) -> None: ...
    def debug(self, message: str) -> None: ...
    def info(self, message: str) -> None: ...
    def warning(self, message: str, *, once: bool = ..., only_once: bool = ...) -> None: ...
    def error(self, message: str) -> None: ...
    def stdout(self, message: str) -> None: ...
    def stderr(self, message: str) -> None: ...

@type_check_only
class _RetrySleepFunctions(TypedDict, total=False):
    default: Callable[[int], int]
    file_access: Callable[[int], int]
    fragment: Callable[[int], int]

@type_check_only
class _ProgressTemplateValue(TypedDict, total=False):  # noqa: Y049
    info: str
    progress: str

@type_check_only
class _ExternalDownloader(TypedDict, total=False):
    dash: str
    default: str
    ftp: str
    http: str
    m3u8: str
    mms: str
    rtmp: str
    rtsp: str

@type_check_only
class _DownloadRange(TypedDict):
    end_time: int
    index: NotRequired[int]
    start_time: int
    title: NotRequired[str]

@type_check_only
class _Color(TypedDict, total=False):
    stderr: Literal["always", "auto", "no_color", "never"]
    stdout: Literal["always", "auto", "no_color", "never"]

_ProgressTemplate = TypedDict(
    "_ProgressTemplate",
    {
        "download": _ProgressTemplateValue,
        "download-title": _ProgressTemplateValue,
        "postprocess": _ProgressTemplateValue,
        "postprocess-title": _ProgressTemplateValue,
    },
)

@type_check_only
class _Params(TypedDict, total=False):
    usenetrc: bool | None
    netrc_location: str | None
    netrc_cmd: str | None
    username: str | None
    password: str | None
    twofactor: str | None
    videopassword: str | None
    ap_mso: str | None
    ap_username: str | None
    ap_password: str | None
    client_certificate: str | None
    client_certificate_key: str | None
    client_certificate_password: str | None
    quiet: bool | None
    no_warnings: bool | None
    forceurl: bool | None
    forcetitle: str | None
    forceid: bool | None
    forcethumbnail: bool | None
    forcedescription: bool | None
    forceduration: str | None
    forcefilename: bool | None
    forceprint: Mapping[str, Collection[str]] | Collection[str] | None
    print_to_file: Mapping[str, tuple[str, str]] | None
    forcejson: bool | None
    dump_single_json: bool | None
    force_write_download_archive: str | None
    simulate: str | None
    skip_download: str | None
    format: str | Callable[[Mapping[str, Any]], Mapping[str, Any]] | None
    allow_unplayable_formats: bool | None
    ignore_no_formats_error: bool | None
    format_sort: Collection[str] | None
    format_sort_force: str | None
    allow_multiple_video_streams: bool | None
    allow_multiple_audio_streams: bool | None
    check_formats: bool | Literal["selected"] | None
    listformats: bool | None
    outtmpl: str | Mapping[str, str] | None
    outtmpl_na_placeholder: str | None
    paths: str | None
    restrictfilenames: bool | None
    windowsfilenames: bool | None
    ignoreerrors: bool | Literal["only_download"] | None
    force_generic_extractor: bool | None
    allowed_extractors: Collection[str] | None
    ratelimit: int | None
    throttledratelimit: int | None
    overwrites: bool | None
    retries: int | None
    file_access_retries: int | None
    fragment_retries: int | None
    extractor_retries: int | None
    retry_sleep_functions: _RetrySleepFunctions | None
    skip_unavailable_fragments: bool | None
    keep_fragments: bool | None
    concurrent_fragment_downloads: int | None
    buffersize: int | None
    noresizebuffer: bool | None
    http_chunk_size: int | None
    continuedl: bool | None
    noprogress: bool | None
    progress_with_newline: bool | None
    progress_template: _ProgressTemplate | None
    playliststart: int | None
    playlistend: int | None
    playlistreverse: bool | None
    playlistrandom: bool | None
    lazy_playlist: bool | None
    noplaylist: bool | None
    logtostderr: bool | None
    consoletitle: str | None
    nopart: bool | None
    updatetime: bool | None
    writedescription: bool | None
    writeinfojson: bool | None
    allow_playlist_files: bool | None
    clean_infojson: bool | None
    getcomments: bool | None
    writethumbnail: bool | None
    write_all_thumbnails: bool | None
    writelink: bool | None
    writeurllink: bool | None
    writewebloclink: bool | None
    writedesktoplink: bool | None
    writesubtitles: bool | None
    writeautomaticsub: bool | None
    allsubtitles: bool | None
    listsubtitles: bool | None
    subtitlesformat: str | None
    subtitleslangs: Collection[str] | None
    matchtitle: bool | None
    rejecttitle: bool | None
    prefer_free_formats: bool | None
    trim_file_name: int | None
    verbose: bool | None
    test: bool | None
    keepvideo: str | None
    min_filesize: int | None
    max_filesize: int | None
    min_views: str | None
    max_views: str | None
    daterange: str | None
    cachedir: str | None
    age_limit: str | None
    download_archive: str | None
    break_on_existing: str | None
    break_on_reject: bool | None
    break_per_url: bool | None
    skip_playlist_after_errors: bool | None
    cookiefile: str | None
    cookiesfrombrowser: tuple[str, ...] | None
    legacyserverconnect: bool | None
    nocheckcertificate: bool | None
    prefer_insecure: str | None
    enable_file_urls: str | None
    http_headers: Mapping[str, str] | None
    proxy: str | None
    socket_timeout: int | None
    bidi_workaround: bool | None
    debug_printtraffic: bool | None
    default_search: str | None
    dynamic_mpd: bool | None
    extractor_args: Mapping[str, Mapping[str, Any]] | None
    js_runtimes: dict[str, dict[str, str] | None]
    remote_components: set[Literal["ejs:npm", "ejs:github"]]
    encoding: str | None
    extract_flat: bool | Literal["in_playlist", "discard", "discard_in_playlist"] | None
    live_from_start: bool | None
    warn_when_outdated: bool | None
    wait_for_video: tuple[int, int] | None
    mark_watched: bool | None
    merge_output_format: str | None
    final_ext: str | None
    postprocessors: Collection[Mapping[str, Any]]
    fixup: Literal["never", "warn", "detect_or_warn"] | None
    source_address: str | None
    sleep_interval_requests: int | None
    sleep_interval: int | None
    max_sleep_interval: int | None
    sleep_interval_subtitles: int | None
    external_downloader: _ExternalDownloader | None
    download_ranges: Callable[[Any, YoutubeDL], Iterator[_DownloadRange]] | None
    force_keyframes_at_cuts: bool | None
    list_thumbnails: str | None
    playlist_items: Collection[int] | None
    match_filter: NotRequired[Callable[[Mapping[str, Any], bool], str | None] | Callable[[Mapping[str, Any]], str | None] | None]
    color: _Color | None
    ffmpeg_location: str | None
    hls_prefer_native: bool | None
    hls_use_mpegts: bool | None
    hls_split_discontinuity: bool | None
    max_downloads: int | None
    dump_intermediate_pages: bool | None
    listformats_table: bool | None
    write_pages: bool | None
    external_downloader_args: Literal["default"] | Mapping[str, Collection[str]] | Collection[str] | None
    postprocessor_args: Mapping[str, Collection[str]] | Collection[str] | None
    geo_verification_proxy: str | None
    geo_bypass: bool | None
    geo_bypass_country: str | None
    geo_bypass_ip_block: str | None
    useid: bool | None
    compat_opts: dict[str, Any] | None
    logger: _LoggerProtocol
    # Undocumented fields below.
    _deprecation_warnings: Collection[str] | None
    _warnings: Collection[str] | None
    autonumber_size: int | None
    autonumber_start: int | None
    load_pages: bool | None
    progress_hooks: list[Callable[[Mapping[str, Any]], object]]
    impersonate: ImpersonateTarget

@type_check_only
class _ParsedOptions(NamedTuple):
    parser: optparse.OptionParser
    options: optparse.Values
    urls: Collection[str]
    ydl_opts: _Params

def parse_options(argv: Collection[str] | None = None) -> _ParsedOptions: ...
def main(argv: list[str] | None = None) -> int: ...



from _typeshed import Incomplete

from authlib.oauth2.rfc8414 import AuthorizationServerMetadata

class OpenIDProviderMetadata(AuthorizationServerMetadata):
    REGISTRY_KEYS: Incomplete
    def validate_jwks_uri(self): ...
    def validate_acr_values_supported(self) -> None: ...
    def validate_subject_types_supported(self) -> None: ...
    def validate_id_token_signing_alg_values_supported(self) -> None: ...
    def validate_id_token_encryption_alg_values_supported(self) -> None: ...
    def validate_id_token_encryption_enc_values_supported(self) -> None: ...
    def validate_userinfo_signing_alg_values_supported(self) -> None: ...
    def validate_userinfo_encryption_alg_values_supported(self) -> None: ...
    def validate_userinfo_encryption_enc_values_supported(self) -> None: ...
    def validate_request_object_signing_alg_values_supported(self) -> None: ...
    def validate_request_object_encryption_alg_values_supported(self) -> None: ...
    def validate_request_object_encryption_enc_values_supported(self) -> None: ...
    def validate_display_values_supported(self) -> None: ...
    def validate_claim_types_supported(self) -> None: ...
    def validate_claims_supported(self) -> None: ...
    def validate_claims_locales_supported(self) -> None: ...
    def validate_claims_parameter_supported(self) -> None: ...
    def validate_request_parameter_supported(self) -> None: ...
    def validate_request_uri_parameter_supported(self) -> None: ...
    def validate_require_request_uri_registration(self) -> None: ...
    @property
    def claim_types_supported(self): ...
    @property
    def claims_parameter_supported(self): ...
    @property
    def request_parameter_supported(self): ...
    @property
    def request_uri_parameter_supported(self): ...
    @property
    def require_request_uri_registration(self): ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import Literal

class Differ:
    F: float
    uniqueattrs: Incomplete
    fast_match: Incomplete
    best_match: Incomplete
    ignored_attrs: Incomplete
    def __init__(
        self,
        F: float | None = None,
        uniqueattrs=None,
        ratio_mode: Literal["accurate", "fast", "faster"] = "fast",
        fast_match: bool = False,
        best_match: bool = False,
        ignored_attrs=[],
    ) -> None: ...
    left: Incomplete
    right: Incomplete
    def clear(self) -> None: ...
    def set_trees(self, left, right) -> None: ...
    def append_match(self, lnode, rnode, max_match) -> None: ...
    def match(self, left=None, right=None): ...
    def node_ratio(self, left, right): ...
    def node_text(self, node): ...
    def node_attribs(self, node): ...
    def leaf_ratio(self, left, right): ...
    def child_ratio(self, left, right): ...
    def update_node_tag(self, left, right) -> Generator[Incomplete]: ...
    def update_node_attr(self, left, right) -> Generator[Incomplete]: ...
    def update_node_text(self, left, right) -> Generator[Incomplete]: ...
    def find_pos(self, node): ...
    def align_children(self, left, right) -> Generator[Incomplete, None, Incomplete]: ...
    def diff(self, left=None, right=None) -> Generator[Incomplete, Incomplete]: ...



from _typeshed import SupportsItems

def parse_colors(path: str) -> SupportsItems[str, str | tuple[int, int, int]]: ...
def parse_rgb_txt_file(path: str) -> SupportsItems[str, str | tuple[int, int, int]]: ...
def parse_json_color_file(path: str) -> dict[str, str]: ...
def sanitize_color_palette(colorpalette: SupportsItems[str, str | tuple[int, int, int]]) -> dict[str, tuple[int, int, int]]: ...



from pathlib import Path
from typing import Any, Literal

from PIL import Image

from .._types import Writeable
from . import base

class PilImage(base.BaseImage):
    kind: Literal["PNG"]
    fill_color: str
    # the new_image and get_image methods accept arbitrary keyword arguments to
    # accommodate subclasses with additional arguments.
    def new_image(self, *, back_color: str = "white", fill_color: str = "black", **kwargs: Any) -> Image.Image: ...
    def get_image(self, **kwargs: Any) -> Image.Image: ...
    def drawrect(self, row: int, col: int) -> None: ...
    # kwargs are passed on to PIL.Image.save, which also accepts arbitrary keyword arguments.
    def save(  # type: ignore[override]
        self,
        stream: str | bytes | Path | Writeable,
        format: str | None = None,
        *,
        kind: str | None = None,
        save_all: bool = ...,
        bitmap_format: Literal["bmp", "png"] = ...,
        optimize: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    # attribute access is forwarded to the wrapped PIL.Image.Image instance.
    def __getattr__(self, name: str) -> Any: ...



from collections.abc import Mapping, Sequence
from io import BytesIO
from re import Pattern

from waitress.adjustments import Adjustments
from waitress.receiver import ChunkedReceiver, FixedStreamReceiver
from waitress.utilities import Error

def unquote_bytes_to_wsgi(bytestring: str | bytes | bytearray) -> str: ...

class ParsingError(Exception): ...
class TransferEncodingNotImplemented(Exception): ...

class HTTPRequestParser:
    completed: bool
    empty: bool
    expect_continue: bool
    headers_finished: bool
    header_plus: bytes
    chunked: bool
    content_length: int
    header_bytes_received: int
    body_bytes_received: int
    body_rcv: ChunkedReceiver | FixedStreamReceiver | None
    version: str
    error: Error | None
    connection_close: bool
    headers: Mapping[str, str]
    adj: Adjustments
    def __init__(self, adj: Adjustments) -> None: ...
    def received(self, data: bytes) -> int: ...
    first_line: str
    command: bytes
    url_scheme: str
    def parse_header(self, header_plus: bytes) -> None: ...
    def get_body_stream(self) -> BytesIO: ...
    def close(self) -> None: ...

def split_uri(uri: bytes) -> tuple[str, str, bytes, str, str]: ...
def get_header_lines(header: bytes) -> Sequence[bytes]: ...

first_line_re: Pattern[str]

def crack_first_line(line: str) -> tuple[bytes, bytes, bytes]: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import Final, NamedTuple

DIFF_NS: Final[str]
DIFF_PREFIX: Final[str]
INSERT_NAME: Final[Incomplete]
DELETE_NAME: Final[Incomplete]
REPLACE_NAME: Final[Incomplete]
RENAME_NAME: Final[Incomplete]
WS_BOTH: Final[int]
WS_TEXT: Final[int]
WS_TAGS: Final[int]
WS_NONE: Final[int]
T_OPEN: Final[int]
T_CLOSE: Final[int]
T_SINGLE: Final[int]
PLACEHOLDER_START: Final[int]

class BaseFormatter:
    def __init__(self, normalize=1, pretty_print: bool = False) -> None: ...
    def prepare(self, left_tree, right_tree) -> None: ...
    def format(self, diff, orig_tree) -> None: ...

class PlaceholderEntry(NamedTuple):
    element: Incomplete
    ttype: Incomplete
    close_ph: Incomplete

class PlaceholderMaker:
    text_tags: Incomplete
    formatting_tags: Incomplete
    placeholder2tag: Incomplete
    tag2placeholder: Incomplete
    placeholder: Incomplete
    diff_tags: Incomplete
    def __init__(self, text_tags=(), formatting_tags=()) -> None: ...
    def get_placeholder(self, element, ttype, close_ph): ...
    def is_placeholder(self, char): ...
    def is_formatting(self, element): ...
    def do_element(self, element) -> None: ...
    def do_tree(self, tree) -> None: ...
    def split_string(self, text): ...
    def undo_string(self, text): ...
    def undo_element(self, elem) -> None: ...
    def undo_tree(self, tree) -> None: ...
    def mark_diff(self, ph, action, attributes=None): ...
    def wrap_diff(self, text, action, attributes=None): ...

class XMLFormatter(BaseFormatter):
    normalize: Incomplete
    pretty_print: Incomplete
    text_tags: Incomplete
    formatting_tags: Incomplete
    use_replace: Incomplete
    placeholderer: Incomplete
    def __init__(
        self, normalize=0, pretty_print: bool = True, text_tags=(), formatting_tags=(), use_replace: bool = False
    ) -> None: ...
    def prepare(self, left_tree, right_tree) -> None: ...
    def finalize(self, result_tree) -> None: ...
    def format(self, diff, orig_tree, differ=None): ...
    def render(self, result): ...
    def handle_action(self, action, result) -> None: ...

class DiffFormatter(BaseFormatter):
    normalize: Incomplete
    def __init__(self, normalize=1, pretty_print: bool = False) -> None: ...
    def prepare(self, left, right) -> None: ...
    def finalize(self, left, right) -> None: ...
    def format(self, diff, orig_tree): ...
    def handle_action(self, action): ...

class XmlDiffFormatter(BaseFormatter):
    normalize: Incomplete
    def __init__(self, normalize=1, pretty_print: bool = False) -> None: ...
    def prepare(self, left, right) -> None: ...
    def finalize(self, left, right) -> None: ...
    def format(self, diff, orig_tree): ...
    def handle_action(self, action, orig_tree) -> Generator[Incomplete, Incomplete]: ...



from _typeshed import Unused
from collections.abc import Iterable
from logging import Logger

from oauthlib.common import Request, _HTTPMethod

log: Logger

def signature_base_string(http_method: _HTTPMethod, base_str_uri: str, normalized_encoded_request_parameters: str) -> str: ...
def base_string_uri(uri: str, host: str | None = None) -> str: ...
def collect_parameters(
    uri_query: str = "",
    body: str | bytes | dict[str, str] | Iterable[tuple[str, str]] | None = None,
    headers: dict[str, str] | None = None,
    exclude_oauth_signature: bool = True,
    with_realm: bool = False,
) -> list[tuple[str, str]]: ...
def normalize_parameters(params: dict[str, str]) -> str: ...
def sign_hmac_sha1_with_client(sig_base_str: str, client): ...
def verify_hmac_sha1(request: Request, client_secret=None, resource_owner_secret=None) -> bool: ...
def sign_hmac_sha1(base_string: str | bytes, client_secret, resource_owner_secret): ...
def sign_hmac_sha256_with_client(sig_base_str, client): ...
def verify_hmac_sha256(request, client_secret=None, resource_owner_secret=None) -> bool: ...
def sign_hmac_sha256(base_string: str | bytes, client_secret, resource_owner_secret): ...
def sign_hmac_sha512_with_client(sig_base_str: str, client): ...
def verify_hmac_sha512(request, client_secret: str | None = None, resource_owner_secret: str | None = None) -> bool: ...
def sign_rsa_sha1_with_client(sig_base_str: str | bytes, client): ...
def verify_rsa_sha1(request, rsa_public_key: str) -> bool: ...
def sign_rsa_sha1(base_string, rsa_private_key): ...
def sign_rsa_sha256_with_client(sig_base_str: str, client): ...
def verify_rsa_sha256(request, rsa_public_key: str) -> bool: ...
def sign_rsa_sha512_with_client(sig_base_str: str, client): ...
def verify_rsa_sha512(request, rsa_public_key: str) -> bool: ...
def sign_plaintext_with_client(_signature_base_string: Unused, client) -> str: ...
def sign_plaintext(client_secret: str | None, resource_owner_secret: str | None) -> str: ...
def verify_plaintext(request, client_secret: str | None = None, resource_owner_secret: str | None = None) -> bool: ...



from collections.abc import Callable
from datetime import datetime, tzinfo

from dateparser.conf import Settings

class DateParser:
    def parse(
        self,
        date_string: str,
        parse_method: Callable[[str, Settings, tzinfo | None], tuple[datetime, str]],
        settings: Settings | None = None,
    ) -> tuple[datetime, str]: ...

date_parser: DateParser



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import Final, Literal, TypedDict, TypeVar, overload

from .healthcheck import Healthcheck

_T = TypeVar("_T")

class TaskTemplate(dict[str, Incomplete]):
    def __init__(
        self,
        container_spec: ContainerSpec,
        resources: Resources | None = None,
        restart_policy: RestartPolicy | None = None,
        placement: Placement | list[str] | None = None,
        log_driver: DriverConfig | None = None,
        networks: Iterable[str | NetworkAttachmentConfig] | None = None,
        force_update: int | None = None,
    ) -> None: ...
    @property
    def container_spec(self) -> ContainerSpec: ...
    @property
    def resources(self) -> Resources: ...
    @property
    def restart_policy(self) -> RestartPolicy: ...
    @property
    def placement(self) -> Placement: ...

class ContainerSpec(dict[str, Incomplete]):
    def __init__(
        self,
        image: str,
        command: str | list[str] | None = None,
        args: list[str] | None = None,
        hostname: str | None = None,
        env: dict[str, Incomplete] | list[str] | None = None,
        workdir: str | None = None,
        user: str | None = None,
        labels: dict[str, str] | None = None,
        mounts: Iterable[str | Mount] | None = None,
        stop_grace_period: int | None = None,
        secrets: list[SecretReference] | None = None,
        tty: bool | None = None,
        groups: list[str] | None = None,
        open_stdin: bool | None = None,
        read_only: bool | None = None,
        stop_signal: str | None = None,
        healthcheck: Healthcheck | None = None,
        hosts: Mapping[str, str] | None = None,
        dns_config: DNSConfig | None = None,
        configs: list[ConfigReference] | None = None,
        privileges: Privileges | None = None,
        isolation: str | None = None,
        init: bool | None = None,
        cap_add: list[str] | None = None,
        cap_drop: list[str] | None = None,
        sysctls: dict[str, str] | None = None,
    ) -> None: ...

class Mount(dict[str, Incomplete]):
    def __init__(
        self,
        target: str,
        source: str | None,
        type: Literal["bind", "volume", "tmpfs", "npipe"] = "volume",
        read_only: bool = False,
        consistency: Literal["default", "consistent", "cached", "delegated"] | None = None,
        propagation: str | None = None,
        no_copy: bool = False,
        labels: dict[str, str] | None = None,
        driver_config: DriverConfig | None = None,
        tmpfs_size: int | str | None = None,
        tmpfs_mode: int | None = None,
    ) -> None: ...
    @classmethod
    def parse_mount_string(cls, string: str) -> Mount: ...

class _ResourceDict(TypedDict):
    Kind: str
    Value: int

class Resources(dict[str, Incomplete]):
    def __init__(
        self,
        cpu_limit: int | None = None,
        mem_limit: int | None = None,
        cpu_reservation: int | None = None,
        mem_reservation: int | None = None,
        generic_resources: (
            dict[str, int | str] | list[dict[Literal["DiscreteResourceSpec", "NamedResourceSpec"], _ResourceDict]] | None
        ) = None,
    ) -> None: ...

class UpdateConfig(dict[str, Incomplete]):
    def __init__(
        self,
        parallelism: int = 0,
        delay: int | None = None,
        failure_action: Literal["pause", "continue", "rollback"] = "continue",
        monitor: int | None = None,
        max_failure_ratio: float | None = None,
        order: Literal["start-first", "stop-first"] | None = None,
    ) -> None: ...

class RollbackConfig(UpdateConfig): ...

class RestartConditionTypesEnum:
    NONE: Final = "none"
    ON_FAILURE: Final = "on-failure"
    ANY: Final = "any"

class RestartPolicy(dict[str, Incomplete]):
    condition_types: type[RestartConditionTypesEnum]
    def __init__(
        self, condition: Literal["none", "on-failure", "any"] = "none", delay: int = 0, max_attempts: int = 0, window: int = 0
    ) -> None: ...

class DriverConfig(dict[str, Incomplete]):
    def __init__(self, name: str, options: dict[str, str] | None = None) -> None: ...

class EndpointSpec(dict[str, Incomplete]):
    def __init__(
        self, mode: str | None = None, ports: Mapping[str, str | tuple[str | None, ...]] | list[dict[str, str]] | None = None
    ) -> None: ...

@overload
def convert_service_ports(ports: list[_T]) -> list[_T]: ...
@overload
def convert_service_ports(ports: Mapping[str, str | tuple[str | None, ...]]) -> list[dict[str, str]]: ...

class ServiceMode(dict[str, Incomplete]):
    mode: Literal["replicated", "global", "ReplicatedJob", "GlobalJob"]
    def __init__(
        self,
        mode: Literal["replicated", "global", "replicated-job", "global-job"],
        replicas: int | None = None,
        concurrency: int | None = None,
    ) -> None: ...
    @property
    def replicas(self) -> int | None: ...

class SecretReference(dict[str, Incomplete]):
    def __init__(
        self,
        secret_id: str,
        secret_name: str,
        filename: str | None = None,
        uid: str | None = None,
        gid: str | None = None,
        mode: int = 292,
    ) -> None: ...

class ConfigReference(dict[str, Incomplete]):
    def __init__(
        self,
        config_id: str,
        config_name: str,
        filename: str | None = None,
        uid: str | None = None,
        gid: str | None = None,
        mode: int = 292,
    ) -> None: ...

class Placement(dict[str, Incomplete]):
    def __init__(
        self,
        constraints: list[str] | None = None,
        preferences: Iterable[tuple[str, str] | PlacementPreference] | None = None,
        platforms: Iterable[tuple[str, str]] | None = None,
        maxreplicas: int | None = None,
    ) -> None: ...

class PlacementPreference(dict[str, Incomplete]):
    def __init__(self, strategy: Literal["spread"], descriptor: str) -> None: ...

class DNSConfig(dict[str, Incomplete]):
    def __init__(
        self, nameservers: list[str] | None = None, search: list[str] | None = None, options: list[str] | None = None
    ) -> None: ...

class Privileges(dict[str, Incomplete]):
    def __init__(
        self,
        credentialspec_file: str | None = None,
        credentialspec_registry: str | None = None,
        selinux_disable: bool | None = None,
        selinux_user: str | None = None,
        selinux_role: str | None = None,
        selinux_type: str | None = None,
        selinux_level: str | None = None,
    ) -> None: ...

class NetworkAttachmentConfig(dict[str, Incomplete]):
    def __init__(self, target: str, aliases: list[str] | None = None, options: dict[str, str] | None = None) -> None: ...



from _typeshed import Incomplete
from abc import abstractmethod
from collections.abc import Callable
from typing import IO, Any, Literal, Protocol, type_check_only
from typing_extensions import Self, TypeAlias

from reportlab.pdfgen.canvas import Canvas
from reportlab.platypus.flowables import Flowable
from reportlab.platypus.frames import Frame

__all__ = (
    "ActionFlowable",
    "BaseDocTemplate",
    "CurrentFrameFlowable",
    "FrameActionFlowable",
    "FrameBreak",
    "Indenter",
    "IndexingFlowable",
    "LayoutError",
    "LCActionFlowable",
    "NextFrameFlowable",
    "NextPageTemplate",
    "NotAtTopPageBreak",
    "NullActionFlowable",
    "PageAccumulator",
    "PageBegin",
    "PageTemplate",
    "SimpleDocTemplate",
)

# NOTE: Since we don't know what kind of DocTemplate we will use in a PageTemplate
#       we'll leave the second argument at Any, since the workaround with unbound
#       type vars didn't seem to work for this one
_PageCallback: TypeAlias = Callable[[Canvas, Any], object]

@type_check_only
class _CanvasMaker(Protocol):
    # NOTE: This matches a subset of Canvas.__init__
    def __call__(
        self,
        filename: str | IO[bytes],
        /,
        *,
        pagesize=None,
        pageCompression=None,
        invariant=None,
        enforceColorSpace=None,
        initialFontName=None,
        initialFontSize=None,
        initialLeading=None,
        cropBox=None,
        artBox=None,
        trimBox=None,
        bleedBox=None,
        lang=None,
    ) -> Canvas: ...

class LayoutError(Exception): ...

# NOTE: While internal, this is used as sentinel value in PageTemplate
#       and SimpleDocTemplate so in subclasses you may need to use this
def _doNothing(canvas: Canvas, doc: BaseDocTemplate) -> None: ...

class PTCycle(list[PageTemplate]):
    @property
    def next_value(self) -> PageTemplate: ...
    @property
    def peek(self) -> PageTemplate: ...

class IndexingFlowable(Flowable):
    def isIndexing(self) -> Literal[1]: ...
    def isSatisfied(self) -> int: ...
    def notify(self, kind: str, stuff: Any) -> None: ...
    def beforeBuild(self) -> None: ...
    def afterBuild(self) -> None: ...

class ActionFlowable(Flowable):
    # NOTE: Technically action always has to contain a string referencing
    #       a handle_ method on the DocTemplate, while the rest are the args
    #       that should be passed to that method, but since the default arg
    #       on __init__ violates that we might as well keep things simple
    action: tuple[Any, ...]
    def __init__(self, action: list[Any] | tuple[Any, ...] = ()) -> None: ...
    def apply(self, doc: BaseDocTemplate) -> None: ...
    def __call__(self) -> Self: ...

class NullActionFlowable(ActionFlowable): ...

class LCActionFlowable(ActionFlowable):
    locChanger: int
    def draw(self) -> None: ...

class NextFrameFlowable(ActionFlowable):
    locChanger: int
    def __init__(self, ix: int | str, resume: int = 0) -> None: ...

class CurrentFrameFlowable(LCActionFlowable):
    def __init__(self, ix: int | str, resume: int = 0) -> None: ...

class _FrameBreak(LCActionFlowable):
    def __call__(self, ix: int | str | None = None, resume: int = 0) -> Self: ...
    def apply(self, doc: BaseDocTemplate) -> None: ...

FrameBreak: _FrameBreak
PageBegin: LCActionFlowable

class FrameActionFlowable(Flowable):
    @abstractmethod
    def __init__(self, *arg: Any, **kw: Any) -> None: ...
    @abstractmethod
    def frameAction(self, frame: Frame) -> None: ...

class Indenter(FrameActionFlowable):
    width: float
    height: float
    left: float
    right: float
    def __init__(self, left: float | str = 0, right: float | str = 0) -> None: ...
    def frameAction(self, frame: Frame) -> None: ...

class NotAtTopPageBreak(FrameActionFlowable):
    locChanger: int
    nextTemplate: Incomplete
    def __init__(self, nextTemplate=None) -> None: ...
    def frameAction(self, frame: Frame) -> None: ...

class NextPageTemplate(ActionFlowable):
    locChanger: int
    def __init__(self, pt: str | int | list[str] | tuple[str, ...]) -> None: ...

class PageTemplate:
    id: str | None
    frames: list[Frame]
    onPage: _PageCallback
    onPageEnd: _PageCallback
    pagesize: tuple[float, float]
    autoNextPageTemplate: Incomplete
    cropBox: Incomplete
    artBox: Incomplete
    trimBox: Incomplete
    bleedBox: Incomplete
    def __init__(
        self,
        id: str | None = None,
        frames: list[Frame] | Frame = [],
        onPage: _PageCallback = ...,
        onPageEnd: _PageCallback = ...,
        pagesize: tuple[float, float] | None = None,
        autoNextPageTemplate=None,
        cropBox=None,
        artBox=None,
        trimBox=None,
        bleedBox=None,
    ) -> None: ...
    def beforeDrawPage(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...
    def checkPageSize(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...
    def afterDrawPage(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...

class onDrawStr(str):
    onDraw: Callable[[Canvas, str | None, str], object]
    kind: str | None
    label: str
    def __new__(
        cls, value: object, onDraw: Callable[[Canvas, str | None, str], object], label: str, kind: str | None = None
    ) -> Self: ...
    def __getnewargs__(self) -> tuple[str, Callable[[Canvas, str | None, str], object], str, str | None]: ...  # type: ignore[override]

class PageAccumulator:
    name: str
    data: list[tuple[Any, ...]]
    def __init__(self, name: str | None = None) -> None: ...
    def reset(self) -> None: ...
    def add(self, *args) -> None: ...
    def onDrawText(self, *args) -> str: ...
    def __call__(self, canv: Canvas, kind: str | None, label: str) -> None: ...
    def attachToPageTemplate(self, pt: PageTemplate) -> None: ...
    def onPage(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...
    def onPageEnd(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...
    def pageEndAction(self, canv: Canvas, doc: BaseDocTemplate) -> None: ...
    def onDrawStr(self, value: object, *args) -> onDrawStr: ...

class BaseDocTemplate:
    filename: Incomplete
    pagesize: Incomplete
    pageTemplates: list[PageTemplate]
    showBoundary: Incomplete
    width: float
    height: float
    leftMargin: float
    rightMargin: float
    topMargin: float
    bottomMargin: float
    allowSplitting: Incomplete
    title: Incomplete | None
    author: Incomplete | None
    subject: Incomplete | None
    creator: Incomplete | None
    producer: Incomplete | None
    keywords: list[Incomplete]
    invariant: Incomplete | None
    pageCompression: Incomplete | None
    rotation: Incomplete
    encrypt: Incomplete | None
    cropMarks: Incomplete | None
    enforceColorSpace: Incomplete | None
    displayDocTitle: Incomplete | None
    lang: Incomplete | None
    initialFontName: Incomplete | None
    initialFontSize: Incomplete | None
    initialLeading: Incomplete | None
    cropBox: Incomplete | None
    artBox: Incomplete | None
    trimBox: Incomplete | None
    bleedBox: Incomplete | None
    keepTogetherClass: type[Flowable]
    hideToolbar: Incomplete | None
    hideMenubar: Incomplete | None
    hideWindowUI: Incomplete | None
    fitWindow: Incomplete | None
    centerWindow: Incomplete | None
    nonFullScreenPageMode: Incomplete | None
    direction: Incomplete | None
    viewArea: Incomplete | None
    viewClip: Incomplete | None
    printArea: Incomplete | None
    printClip: Incomplete | None
    printScaling: Incomplete | None
    duplex: Incomplete | None
    # NOTE: The following attributes only exist while/after pages are rendered
    pageTemplate: PageTemplate
    page: int
    frame: Frame
    canv: Canvas
    # TODO: Use TypedDict with Unpack for **kw
    def __init__(self, filename: str | IO[bytes], **kw) -> None: ...
    def setPageCallBack(self, func: Callable[[int], object] | None) -> None: ...
    def setProgressCallBack(self, func: Callable[[str, int], object] | None) -> None: ...
    def clean_hanging(self) -> None: ...
    def addPageTemplates(self, pageTemplates: list[PageTemplate] | tuple[PageTemplate, ...] | PageTemplate) -> None: ...
    def handle_documentBegin(self) -> None: ...
    def handle_pageBegin(self) -> None: ...
    def handle_pageEnd(self) -> None: ...
    def handle_pageBreak(self, slow: bool | None = None) -> None: ...
    def handle_frameBegin(self, resume: int = 0, pageTopFlowables=None) -> None: ...
    def handle_frameEnd(self, resume: int = 0) -> None: ...
    def handle_nextPageTemplate(self, pt: str | int | list[str] | tuple[str, ...]) -> None: ...
    def handle_nextFrame(self, fx: str | int, resume: int = 0) -> None: ...
    def handle_currentFrame(self, fx: str | int, resume: int = 0) -> None: ...
    def handle_breakBefore(self, flowables: list[Flowable]) -> None: ...
    def handle_keepWithNext(self, flowables: list[Flowable]) -> None: ...
    def handle_flowable(self, flowables: list[Flowable]) -> None: ...
    def build(
        self, flowables: list[Flowable], filename: str | IO[bytes] | None = None, canvasmaker: _CanvasMaker = ...
    ) -> None: ...
    def notify(self, kind: str, stuff: Any) -> None: ...
    def pageRef(self, label: str) -> None: ...
    def multiBuild(self, story: list[Flowable], maxPasses: int = 10, **buildKwds: Any) -> int: ...
    def afterInit(self) -> None: ...
    def beforeDocument(self) -> None: ...
    def beforePage(self) -> None: ...
    def afterPage(self) -> None: ...
    def filterFlowables(self, flowables: list[Flowable]) -> None: ...
    def afterFlowable(self, flowable: Flowable) -> None: ...
    def docAssign(self, var: str, expr: object, lifetime: str) -> None: ...
    def docExec(self, stmt: str, lifetime: str) -> None: ...
    def docEval(self, expr: str) -> Any: ...

class SimpleDocTemplate(BaseDocTemplate):
    def handle_pageBegin(self) -> None: ...
    def build(  # type: ignore[override]
        self,
        flowables: list[Flowable],
        onFirstPage: _PageCallback = ...,
        onLaterPages: _PageCallback = ...,
        canvasmaker: _CanvasMaker = ...,
    ) -> None: ...



from hvac.api.vault_api_base import VaultApiBase

class AppRole(VaultApiBase):
    def create_or_update_approle(
        self,
        role_name,
        bind_secret_id=None,
        secret_id_bound_cidrs=None,
        secret_id_num_uses=None,
        secret_id_ttl=None,
        enable_local_secret_ids=None,
        token_ttl=None,
        token_max_ttl=None,
        token_policies=None,
        token_bound_cidrs=None,
        token_explicit_max_ttl=None,
        token_no_default_policy=None,
        token_num_uses=None,
        token_period=None,
        token_type=None,
        mount_point="approle",
    ): ...
    def list_roles(self, mount_point="approle"): ...
    def read_role(self, role_name, mount_point="approle"): ...
    def delete_role(self, role_name, mount_point="approle"): ...
    def read_role_id(self, role_name, mount_point="approle"): ...
    def update_role_id(self, role_name, role_id, mount_point="approle"): ...
    def generate_secret_id(
        self, role_name, metadata=None, cidr_list=None, token_bound_cidrs=None, mount_point="approle", wrap_ttl=None
    ): ...
    def create_custom_secret_id(
        self, role_name, secret_id, metadata=None, cidr_list=None, token_bound_cidrs=None, mount_point="approle", wrap_ttl=None
    ): ...
    def read_secret_id(self, role_name, secret_id, mount_point="approle"): ...
    def destroy_secret_id(self, role_name, secret_id, mount_point="approle"): ...
    def list_secret_id_accessors(self, role_name, mount_point="approle"): ...
    def read_secret_id_accessor(self, role_name, secret_id_accessor, mount_point="approle"): ...
    def destroy_secret_id_accessor(self, role_name, secret_id_accessor, mount_point="approle"): ...
    def login(self, role_id, secret_id=None, use_token: bool = True, mount_point="approle"): ...



from _typeshed import Incomplete
from collections.abc import Collection, Mapping

from authlib.jose.rfc7517 import Key, KeySet

class JsonWebKey:
    JWK_KEY_CLS: Incomplete
    @classmethod
    def generate_key(cls, kty, crv_or_size, options=None, is_private: bool = False): ...
    @classmethod
    def import_key(cls, raw: Mapping[str, object], options: Mapping[str, object] | None = None) -> Key: ...
    @classmethod
    def import_key_set(cls, raw: str | Collection[str] | dict[str, object]) -> KeySet: ...



from typing import Final

__version__: Final[str]

def rectCorner(x, y, width, height, anchor: str = "sw", dims: bool = False): ...
def aspectRatioFix(preserve, anchor, x, y, width, height, imWidth, imHeight, anchorAtXY: bool = False): ...



from typing import ClassVar, Final, TypeVar

from docutils import nodes, readers

__docformat__: Final = "reStructuredText"

_S = TypeVar("_S", bound=str | bytes)

class Reader(readers.Reader[_S]):
    document: nodes.document | None  # type: ignore[assignment]
    config_section_dependencies: ClassVar[tuple[str, ...]]



from __future__ import annotations

import sys
from collections.abc import Generator, Iterable
from contextlib import contextmanager
from typing import Any, NamedTuple

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib

from ts_utils.metadata import StubtestSettings, metadata_path
from ts_utils.utils import NamedTemporaryFile, TemporaryFileWrapper


class MypyDistConf(NamedTuple):
    module_name: str
    values: dict[str, dict[str, Any]]


# The configuration section in the metadata file looks like the following, with multiple module sections possible
# [mypy-tests]
# [mypy-tests.yaml]
# module_name = "yaml"
# [mypy-tests.yaml.values]
# disallow_incomplete_defs = true
# disallow_untyped_defs = true


def mypy_configuration_from_distribution(distribution: str) -> list[MypyDistConf]:
    with metadata_path(distribution).open("rb") as f:
        data = tomllib.load(f)

    # TODO: This could be added to ts_utils.metadata
    mypy_tests_conf: dict[str, dict[str, Any]] = data.get("mypy-tests", {})
    if not mypy_tests_conf:
        return []

    def validate_configuration(section_name: str, mypy_section: dict[str, Any]) -> MypyDistConf:
        assert isinstance(mypy_section, dict), f"{section_name} should be a section"
        module_name = mypy_section.get("module_name")

        assert module_name is not None, f"{section_name} should have a module_name key"
        assert isinstance(module_name, str), f"{section_name} should be a key-value pair"

        assert "values" in mypy_section, f"{section_name} should have a values section"
        values: dict[str, dict[str, Any]] = mypy_section["values"]
        assert isinstance(values, dict), "values should be a section"
        return MypyDistConf(module_name, values.copy())

    assert isinstance(mypy_tests_conf, dict), "mypy-tests should be a section"
    return [validate_configuration(section_name, mypy_section) for section_name, mypy_section in mypy_tests_conf.items()]


@contextmanager
def temporary_mypy_config_file(
    configurations: Iterable[MypyDistConf], stubtest_settings: StubtestSettings | None = None
) -> Generator[TemporaryFileWrapper[str]]:
    temp = NamedTemporaryFile("w+")
    try:
        for dist_conf in configurations:
            temp.write(f"[mypy-{dist_conf.module_name}]\n")
            for k, v in dist_conf.values.items():
                temp.write(f"{k} = {v}\n")
        temp.write("[mypy]\n")

        if stubtest_settings:
            if stubtest_settings.mypy_plugins:
                temp.write(f"plugins = {'.'.join(stubtest_settings.mypy_plugins)}\n")

            if stubtest_settings.mypy_plugins_config:
                for plugin_name, plugin_dict in stubtest_settings.mypy_plugins_config.items():
                    temp.write(f"[mypy.plugins.{plugin_name}]\n")
                    for k, v in plugin_dict.items():
                        temp.write(f"{k} = {v}\n")

        temp.flush()
        yield temp
    finally:
        temp.close()



from _typeshed import Incomplete
from collections.abc import Callable
from logging import Logger

from oauthlib.common import Request
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator as OAuth2RequestValidator

log: Logger

class RequestValidator(OAuth2RequestValidator):
    def get_authorization_code_scopes(self, client_id: str, code: str, redirect_uri: str, request) -> list[str]: ...
    def get_authorization_code_nonce(self, client_id: str, code: str, redirect_uri: str, request) -> str: ...
    def get_jwt_bearer_token(self, token: dict[str, Incomplete], token_handler, request: Request) -> str: ...
    def get_id_token(self, token: dict[str, Incomplete], token_handler, request: Request) -> str: ...
    def finalize_id_token(
        self, id_token: dict[str, Incomplete], token: dict[str, Incomplete], token_handler: Callable[..., str], request: Request
    ) -> str: ...
    def validate_jwt_bearer_token(self, token: str, scopes, request: Request) -> bool: ...
    def validate_id_token(self, token: str, scopes, request: Request) -> bool: ...
    def validate_silent_authorization(self, request: Request) -> bool: ...
    def validate_silent_login(self, request: Request) -> bool: ...
    def validate_user_match(self, id_token_hint: str, scopes, claims: dict[str, Incomplete], request: Request) -> bool: ...
    def get_userinfo_claims(self, request: Request) -> dict[str, Incomplete] | str: ...
    def refresh_id_token(self, request: Request) -> bool: ...



from __future__ import annotations

from typing import Protocol, cast
from typing_extensions import assert_type

import grpc


class DummyRequest:
    pass


class DummyReply:
    pass


class DummyServiceStub(Protocol):
    UnaryUnary: grpc.UnaryUnaryMultiCallable[DummyRequest, DummyReply]
    UnaryStream: grpc.UnaryStreamMultiCallable[DummyRequest, DummyReply]
    StreamUnary: grpc.StreamUnaryMultiCallable[DummyRequest, DummyReply]
    StreamStream: grpc.StreamStreamMultiCallable[DummyRequest, DummyReply]


stub = cast(DummyServiceStub, None)
req = DummyRequest()

assert_type(stub.UnaryUnary(req), DummyReply)

for resp in stub.UnaryStream(req):
    assert_type(resp, DummyReply)

assert_type(stub.StreamUnary(iter([req])), DummyReply)

for resp in stub.StreamStream(iter([req])):
    assert_type(resp, DummyReply)



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Callable, Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "graph_edit_distance",
    "optimal_edit_paths",
    "optimize_graph_edit_distance",
    "optimize_edit_paths",
    "simrank_similarity",
    "panther_similarity",
    "generate_random_paths",
]

@_dispatchable
def graph_edit_distance(
    G1: Graph[_Node],
    G2: Graph[_Node],
    node_match: Callable[..., Incomplete] | None = None,
    edge_match: Callable[..., Incomplete] | None = None,
    node_subst_cost: Callable[..., Incomplete] | None = None,
    node_del_cost: Callable[..., Incomplete] | None = None,
    node_ins_cost: Callable[..., Incomplete] | None = None,
    edge_subst_cost: Callable[..., Incomplete] | None = None,
    edge_del_cost: Callable[..., Incomplete] | None = None,
    edge_ins_cost: Callable[..., Incomplete] | None = None,
    roots=None,
    upper_bound: float | None = None,
    timeout: float | None = None,
): ...
@_dispatchable
def optimal_edit_paths(
    G1: Graph[_Node],
    G2: Graph[_Node],
    node_match: Callable[..., Incomplete] | None = None,
    edge_match: Callable[..., Incomplete] | None = None,
    node_subst_cost: Callable[..., Incomplete] | None = None,
    node_del_cost: Callable[..., Incomplete] | None = None,
    node_ins_cost: Callable[..., Incomplete] | None = None,
    edge_subst_cost: Callable[..., Incomplete] | None = None,
    edge_del_cost: Callable[..., Incomplete] | None = None,
    edge_ins_cost: Callable[..., Incomplete] | None = None,
    upper_bound: float | None = None,
): ...
@_dispatchable
def optimize_graph_edit_distance(
    G1: Graph[_Node],
    G2: Graph[_Node],
    node_match: Callable[..., Incomplete] | None = None,
    edge_match: Callable[..., Incomplete] | None = None,
    node_subst_cost: Callable[..., Incomplete] | None = None,
    node_del_cost: Callable[..., Incomplete] | None = None,
    node_ins_cost: Callable[..., Incomplete] | None = None,
    edge_subst_cost: Callable[..., Incomplete] | None = None,
    edge_del_cost: Callable[..., Incomplete] | None = None,
    edge_ins_cost: Callable[..., Incomplete] | None = None,
    upper_bound: float | None = None,
) -> Generator[Incomplete, None, None]: ...
@_dispatchable
def optimize_edit_paths(
    G1: Graph[_Node],
    G2: Graph[_Node],
    node_match: Callable[..., Incomplete] | None = None,
    edge_match: Callable[..., Incomplete] | None = None,
    node_subst_cost: Callable[..., Incomplete] | None = None,
    node_del_cost: Callable[..., Incomplete] | None = None,
    node_ins_cost: Callable[..., Incomplete] | None = None,
    edge_subst_cost: Callable[..., Incomplete] | None = None,
    edge_del_cost: Callable[..., Incomplete] | None = None,
    edge_ins_cost: Callable[..., Incomplete] | None = None,
    upper_bound: float | None = None,
    strictly_decreasing: bool = True,
    roots=None,
    timeout: float | None = None,
) -> Generator[Incomplete, None, Incomplete]: ...
@_dispatchable
def simrank_similarity(
    G: Graph[_Node],
    source: _Node | None = None,
    target: _Node | None = None,
    importance_factor: float = 0.9,
    max_iterations: int = 1000,
    tolerance: float = 0.0001,
) -> float | dict[Incomplete, Incomplete]: ...
@_dispatchable
def panther_similarity(
    G: Graph[_Node],
    source: _Node,
    k: int = 5,
    path_length: int = 5,
    c: float = 0.5,
    delta: float = 0.1,
    eps=None,
    weight: str | None = "weight",
) -> dict[bytes, bytes]: ...
@_dispatchable
def generate_random_paths(
    G: Graph[_Node],
    sample_size: int,
    path_length: int = 5,
    index_map: SupportsGetItem[Incomplete, Incomplete] | None = None,
    weight: str | None = "weight",
    seed: int | RandomState | None = None,
    *,
    source: _Node | None = None,
) -> Generator[list[Incomplete]]: ...



import sys
from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.exception import NetworkXException
from networkx.utils.backends import _dispatchable

__all__ = [
    "is_chordal",
    "find_induced_nodes",
    "chordal_graph_cliques",
    "chordal_graph_treewidth",
    "NetworkXTreewidthBoundExceeded",
    "complete_to_chordal_graph",
]

class NetworkXTreewidthBoundExceeded(NetworkXException): ...

@_dispatchable
def is_chordal(G: Graph[_Node]) -> bool: ...
@_dispatchable
def find_induced_nodes(G: Graph[_Node], s: _Node, t: _Node, treewidth_bound: float = sys.maxsize) -> set[_Node]: ...
@_dispatchable
def chordal_graph_cliques(G: Graph[_Node]) -> Generator[frozenset[_Node], None, None]: ...
@_dispatchable
def chordal_graph_treewidth(G: Graph[_Node]) -> int: ...
@_dispatchable
def complete_to_chordal_graph(G: Graph[_Node]) -> tuple[Incomplete, dict[Incomplete, int]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class VerifierConfig(google.protobuf.message.Message):
    """The config for graph verifiers."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Toggle:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ToggleEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[VerifierConfig._Toggle.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: VerifierConfig._Toggle.ValueType  # 0
        ON: VerifierConfig._Toggle.ValueType  # 1
        OFF: VerifierConfig._Toggle.ValueType  # 2

    class Toggle(_Toggle, metaclass=_ToggleEnumTypeWrapper): ...
    DEFAULT: VerifierConfig.Toggle.ValueType  # 0
    ON: VerifierConfig.Toggle.ValueType  # 1
    OFF: VerifierConfig.Toggle.ValueType  # 2

    VERIFICATION_TIMEOUT_IN_MS_FIELD_NUMBER: builtins.int
    STRUCTURE_VERIFIER_FIELD_NUMBER: builtins.int
    verification_timeout_in_ms: builtins.int
    """Deadline for completion of all verification i.e. all the Toggle ON
    verifiers must complete execution within this time.
    """
    structure_verifier: global___VerifierConfig.Toggle.ValueType
    """Perform structural validation on a tensorflow graph. Default is OFF."""
    def __init__(
        self,
        *,
        verification_timeout_in_ms: builtins.int | None = ...,
        structure_verifier: global___VerifierConfig.Toggle.ValueType | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "structure_verifier", b"structure_verifier", "verification_timeout_in_ms", b"verification_timeout_in_ms"
        ],
    ) -> None: ...

global___VerifierConfig = VerifierConfig



import datetime
from typing import Final, Literal

_JDN_delta: Final[tuple[int, int]]
epoch_1904: Final[datetime.datetime]
epoch_1900: Final[datetime.datetime]
epoch_1900_minus_1: Final[datetime.datetime]
_XLDAYS_TOO_LARGE: Final[tuple[int, int]]
_days_in_month: Final[tuple[None, int, int, int, int, int, int, int, int, int, int, int, int]]

class XLDateError(ValueError): ...
class XLDateNegative(XLDateError): ...
class XLDateAmbiguous(XLDateError): ...
class XLDateTooLarge(XLDateError): ...
class XLDateBadDatemode(XLDateError): ...
class XLDateBadTuple(XLDateError): ...

# 0: 1900-based, 1: 1904-based.
def xldate_as_tuple(xldate: float, datemode: Literal[0, 1]) -> tuple[int, int, int, int, int, int]: ...
def xldate_as_datetime(xldate: float, datemode: Literal[0, 1]) -> datetime.datetime: ...
def _leap(y: int) -> Literal[0, 1]: ...
def xldate_from_date_tuple(date_tuple: tuple[int, int, int], datemode: Literal[0, 1]) -> float: ...
def xldate_from_time_tuple(time_tuple: tuple[int, int, int]) -> float: ...
def xldate_from_datetime_tuple(datetime_tuple: tuple[int, int, int, int, int, int], datemode: Literal[0, 1]) -> float: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.protobuf.meta_graph_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SavedModel(google.protobuf.message.Message):
    """SavedModel is the high level serialization format for TensorFlow Models.
    See [todo: doc links, similar to session_bundle] for more information.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAVED_MODEL_SCHEMA_VERSION_FIELD_NUMBER: builtins.int
    META_GRAPHS_FIELD_NUMBER: builtins.int
    saved_model_schema_version: builtins.int
    """The schema version of the SavedModel instance. Used for versioning when
    making future changes to the specification/implementation. Initial value
    at release will be 1.
    """
    @property
    def meta_graphs(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.protobuf.meta_graph_pb2.MetaGraphDef
    ]:
        """One or more MetaGraphs."""

    def __init__(
        self,
        *,
        saved_model_schema_version: builtins.int | None = ...,
        meta_graphs: collections.abc.Iterable[tensorflow.core.protobuf.meta_graph_pb2.MetaGraphDef] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal["meta_graphs", b"meta_graphs", "saved_model_schema_version", b"saved_model_schema_version"],
    ) -> None: ...

global___SavedModel = SavedModel



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["junction_tree"]

@_dispatchable
def junction_tree(G: Graph[_Node]) -> Graph[Incomplete]: ...



from re import Pattern
from typing import ClassVar
from typing_extensions import deprecated
from xml.etree.ElementTree import Element

from markdown.blockparser import BlockParser
from markdown.blockprocessors import BlockProcessor
from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor
from markdown.treeprocessors import Treeprocessor

class AbbrExtension(Extension):
    def reset(self) -> None: ...
    def reset_glossary(self) -> None: ...
    def load_glossary(self, dictionary: dict[str, str]) -> None: ...

class AbbrTreeprocessor(Treeprocessor):
    RE: Pattern[str] | None
    abbrs: dict[str, str]
    def __init__(self, md: Markdown | None = None, abbrs: dict[str, str] | None = None) -> None: ...
    def create_element(self, title: str, text: str, tail: str) -> Element: ...
    def iter_element(self, el: Element, parent: Element | None = None) -> None: ...

# Techinically it is the same type as `AbbrPreprocessor` just not deprecated.
class AbbrBlockprocessor(BlockProcessor):
    RE: ClassVar[Pattern[str]]
    abbrs: dict[str, str]
    def __init__(self, parser: BlockParser, abbrs: dict[str, str]) -> None: ...

@deprecated("This class will be removed in the future; use `AbbrTreeprocessor` instead.")
class AbbrPreprocessor(AbbrBlockprocessor): ...

@deprecated("This class will be removed in the future; use `AbbrTreeprocessor` instead.")
class AbbrInlineProcessor(InlineProcessor):
    title: str
    def __init__(self, pattern: str, title: str) -> None: ...

def makeExtension(**kwargs) -> AbbrExtension: ...



# SPDX-License-Identifier: MIT

import inspect
import platform
import sys
import threading

from collections.abc import Mapping, Sequence  # noqa: F401
from typing import _GenericAlias


PYPY = platform.python_implementation() == "PyPy"
PY_3_9_PLUS = sys.version_info[:2] >= (3, 9)
PY_3_10_PLUS = sys.version_info[:2] >= (3, 10)
PY_3_11_PLUS = sys.version_info[:2] >= (3, 11)
PY_3_12_PLUS = sys.version_info[:2] >= (3, 12)
PY_3_13_PLUS = sys.version_info[:2] >= (3, 13)
PY_3_14_PLUS = sys.version_info[:2] >= (3, 14)


if PY_3_14_PLUS:  # pragma: no cover
    import annotationlib

    _get_annotations = annotationlib.get_annotations

else:

    def _get_annotations(cls):
        """
        Get annotations for *cls*.
        """
        return cls.__dict__.get("__annotations__", {})


class _AnnotationExtractor:
    """
    Extract type annotations from a callable, returning None whenever there
    is none.
    """

    __slots__ = ["sig"]

    def __init__(self, callable):
        try:
            self.sig = inspect.signature(callable)
        except (ValueError, TypeError):  # inspect failed
            self.sig = None

    def get_first_param_type(self):
        """
        Return the type annotation of the first argument if it's not empty.
        """
        if not self.sig:
            return None

        params = list(self.sig.parameters.values())
        if params and params[0].annotation is not inspect.Parameter.empty:
            return params[0].annotation

        return None

    def get_return_type(self):
        """
        Return the return type if it's not empty.
        """
        if (
            self.sig
            and self.sig.return_annotation is not inspect.Signature.empty
        ):
            return self.sig.return_annotation

        return None


# Thread-local global to track attrs instances which are already being repr'd.
# This is needed because there is no other (thread-safe) way to pass info
# about the instances that are already being repr'd through the call stack
# in order to ensure we don't perform infinite recursion.
#
# For instance, if an instance contains a dict which contains that instance,
# we need to know that we're already repr'ing the outside instance from within
# the dict's repr() call.
#
# This lives here rather than in _make.py so that the functions in _make.py
# don't have a direct reference to the thread-local in their globals dict.
# If they have such a reference, it breaks cloudpickle.
repr_context = threading.local()


def get_generic_base(cl):
    """If this is a generic class (A[str]), return the generic base for it."""
    if cl.__class__ is _GenericAlias:
        return cl.__origin__
    return None



from collections.abc import Sequence
from typing import Literal, TypedDict, type_check_only

from .matching import _Match

@type_check_only
class _Feedback(TypedDict):
    warning: str
    suggestions: list[str]

def get_feedback(score: Literal[0, 1, 2, 3, 4], sequence: Sequence[_Match]) -> _Feedback: ...
def get_match_feedback(match: _Match, is_sole_match: bool) -> _Feedback: ...
def get_dictionary_match_feedback(match: _Match, is_sole_match: bool) -> _Feedback: ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable
from logging import Logger
from re import Pattern
from typing import Final, Literal, NamedTuple, Protocol, TypeVar, overload, type_check_only

from ._fonttools_shims import BasePen, _TTGlyphSet
from .drawing import ClippingPath, PaintedPath
from .fpdf import FPDF
from .image_datastructures import ImageCache

LOGGER: Logger
__pdoc__: dict[str, bool]

@type_check_only
class _HasQualname(Protocol):
    __qualname__: str

_T = TypeVar("_T", bound=_HasQualname)

def force_nodocument(item: _T) -> _T: ...

NUMBER_SPLIT: Final[Pattern[str]]
TRANSFORM_GETTER: Final[Pattern[str]]

class Percent(float): ...

unit_splitter: Pattern[str]
relative_length_units: set[str]
absolute_length_units: dict[str, int]
angle_units: dict[str, float]

def resolve_length(length_str, default_unit: str = "pt"): ...
def resolve_angle(angle_str, default_unit: str = "deg"): ...
def xmlns(space, name): ...
def xmlns_lookup(space, *names): ...
def without_ns(qualified_tag: str) -> str: ...

shape_tags: Incomplete

def svgcolor(colorstr): ...
def convert_stroke_width(incoming): ...
def convert_miterlimit(incoming): ...
def clamp_float(min_val, max_val): ...
def inheritable(value, converter=...): ...
def optional(value, converter=...): ...

svg_attr_map: dict[str, Callable[[Incomplete], tuple[str, Incomplete]]]

def apply_styles(stylable, svg_element) -> None: ...

class ShapeBuilder:
    @overload
    @staticmethod
    def new_path(tag, clipping_path: Literal[True]) -> ClippingPath: ...
    @overload
    @staticmethod
    def new_path(tag, clipping_path: Literal[False] = False) -> PaintedPath: ...
    @overload
    @classmethod
    def rect(cls, tag, clipping_path: Literal[True]) -> ClippingPath: ...
    @overload
    @classmethod
    def rect(cls, tag, clipping_path: Literal[False] = False) -> PaintedPath: ...
    @overload
    @classmethod
    def circle(cls, tag, clipping_path: Literal[True]) -> ClippingPath: ...
    @overload
    @classmethod
    def circle(cls, tag, clipping_path: Literal[False] = False) -> PaintedPath: ...
    @overload
    @classmethod
    def ellipse(cls, tag, clipping_path: Literal[True]) -> ClippingPath: ...
    @overload
    @classmethod
    def ellipse(cls, tag, clipping_path: Literal[False] = False) -> PaintedPath: ...
    @classmethod
    def line(cls, tag) -> PaintedPath: ...
    @classmethod
    def polyline(cls, tag) -> PaintedPath: ...
    @overload
    @classmethod
    def polygon(cls, tag, clipping_path: Literal[True]) -> ClippingPath: ...
    @overload
    @classmethod
    def polygon(cls, tag, clipping_path: Literal[False] = False) -> PaintedPath: ...

def convert_transforms(tfstr): ...

class PathPen(BasePen):
    pdf_path: PaintedPath
    last_was_line_to: bool
    first_is_move: bool | None
    def __init__(self, pdf_path: PaintedPath, glyphSet: _TTGlyphSet | None = ...): ...
    def arcTo(self, rx, ry, rotation, arc, sweep, end) -> None: ...

def svg_path_converter(pdf_path: PaintedPath, svg_path: str) -> None: ...

class SVGObject:
    image_cache: ImageCache | None

    @classmethod
    def from_file(cls, filename, *args, encoding: str = "utf-8", **kwargs): ...
    cross_references: Incomplete
    def __init__(self, svg_text, image_cache: ImageCache | None = None) -> None: ...
    preserve_ar: Incomplete
    width: Incomplete
    height: Incomplete
    viewbox: Incomplete
    def update_xref(self, key: str | None, referenced) -> None: ...
    def extract_shape_info(self, root_tag) -> None: ...
    base_group: Incomplete
    def convert_graphics(self, root_tag) -> None: ...
    def transform_to_page_viewport(self, pdf, align_viewbox: bool = True): ...
    def transform_to_rect_viewport(
        self, scale, width, height, align_viewbox: bool = True, ignore_svg_top_attrs: bool = False
    ): ...
    def draw_to_page(self, pdf: FPDF, x=None, y=None, debug_stream=None) -> None: ...
    def handle_defs(self, defs) -> None: ...
    def build_xref(self, xref): ...
    def build_group(self, group, pdf_group=None): ...
    def build_path(self, path): ...
    def build_shape(self, shape): ...
    def build_clipping_path(self, shape, clip_id): ...
    def apply_clipping_path(self, stylable, svg_element) -> None: ...
    def build_image(self, image) -> SVGImage: ...

class SVGImage(NamedTuple):
    href: str
    x: float
    y: float
    width: float
    height: float
    svg_obj: SVGObject

    def __deepcopy__(self, _memo: Unused) -> SVGImage: ...
    def render(
        self, _gsd_registry: Unused, _style: Unused, last_item, initial_point
    ) -> tuple[Incomplete, Incomplete, Incomplete]: ...
    def render_debug(
        self, gsd_registry: Unused, style: Unused, last_item, initial_point, debug_stream, _pfx: Unused
    ) -> tuple[Incomplete, Incomplete, Incomplete]: ...



from tensorflow.keras import (
    activations as activations,
    callbacks as callbacks,
    constraints as constraints,
    initializers as initializers,
    layers as layers,
    losses as losses,
    metrics as metrics,
    models as models,
    optimizers as optimizers,
    regularizers as regularizers,
)
from tensorflow.keras.models import Model as Model

def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete

from braintree.transaction_line_item import TransactionLineItem

class TransactionLineItemGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def find_all(self, transaction_id: str) -> list[TransactionLineItem]: ...



from collections.abc import Callable

from networkx.algorithms.shortest_paths.weighted import _WeightFunc
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["astar_path", "astar_path_length"]

@_dispatchable
def astar_path(
    G: Graph[_Node],
    source: _Node,
    target: _Node,
    heuristic: Callable[[_Node, _Node], float] | None = None,
    weight: str | _WeightFunc[_Node] | None = "weight",
    *,
    cutoff: float | None = None,
) -> list[_Node]: ...
@_dispatchable
def astar_path_length(
    G: Graph[_Node],
    source: _Node,
    target: _Node,
    heuristic: Callable[[_Node, _Node], float] | None = None,
    weight: str | _WeightFunc[_Node] | None = "weight",
    *,
    cutoff: float | None = None,
) -> float: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable
from itertools import chain
from logging import Logger
from typing import TypeVar
from typing_extensions import TypeAlias

from oauthlib.common import Request

from ..request_validator import RequestValidator
from ..tokens import TokenBase

log: Logger

_T = TypeVar("_T")
_AuthValidator: TypeAlias = Callable[[Request], dict[str, Incomplete]]
_TokenValidator: TypeAlias = Callable[[Request], None]
_CodeModifier: TypeAlias = Callable[[dict[str, str], TokenBase | None, Request | None], dict[str, str]]
_TokenModifier: TypeAlias = Callable[[dict[str, Incomplete], TokenBase | None, Request | None], dict[str, Incomplete]]

class ValidatorsContainer:
    pre_auth: Iterable[_AuthValidator]
    post_auth: Iterable[_AuthValidator]
    pre_token: Iterable[_TokenValidator]
    post_token: Iterable[_TokenValidator]
    def __init__(
        self,
        post_auth: Iterable[_AuthValidator],
        post_token: Iterable[_TokenValidator],
        pre_auth: Iterable[_AuthValidator],
        pre_token: Iterable[_TokenValidator],
    ) -> None: ...
    @property
    def all_pre(self) -> chain[_AuthValidator | _TokenValidator]: ...
    @property
    def all_post(self) -> chain[_AuthValidator | _TokenValidator]: ...

class GrantTypeBase:
    error_uri: str | None
    request_validator: RequestValidator | None
    default_response_mode: str
    refresh_token: bool
    response_types: list[str]
    def __init__(
        self,
        request_validator: RequestValidator | None = None,
        *,
        post_auth: Iterable[_AuthValidator] | None = None,
        post_token: Iterable[_TokenValidator] | None = None,
        pre_auth: Iterable[_AuthValidator] | None = None,
        pre_token: Iterable[_TokenValidator] | None = None,
        **kwargs,
    ) -> None: ...
    def register_response_type(self, response_type: str) -> None: ...
    def register_code_modifier(self, modifier: _CodeModifier) -> None: ...
    def register_token_modifier(self, modifier: _TokenModifier) -> None: ...
    def create_authorization_response(
        self, request: Request, token_handler: TokenBase
    ) -> tuple[dict[str, str], str | None, int | None]: ...
    def create_token_response(
        self, request: Request, token_handler: TokenBase
    ) -> tuple[dict[str, str], str | None, int | None]: ...
    def add_token(self, token: dict[str, _T], token_handler: TokenBase, request: Request) -> dict[str, _T]: ...
    def validate_grant_type(self, request: Request) -> None: ...
    def validate_scopes(self, request: Request) -> None: ...
    def prepare_authorization_response(
        self, request: Request, token: dict[str, Incomplete], headers: dict[str, str], body: str | None, status: int | None
    ) -> tuple[dict[str, str], str | None, int | None]: ...



from typing import Literal, overload

@overload
def wildcard_match(pattern: None, text: str, case_insensitive: bool = True) -> Literal[False]: ...
@overload
def wildcard_match(pattern: str, text: None, case_insensitive: bool = True) -> Literal[False]: ...
@overload
def wildcard_match(pattern: str, text: str, case_insensitive: bool = True) -> bool: ...



from _typeshed import Incomplete
from typing import Any
from typing_extensions import TypeAlias

from django.db.models import Model, QuerySet

from .fields import Field
from .resources import Resource

Dataset: TypeAlias = Incomplete  # tablib.Dataset

class BaseInstanceLoader:
    resource: Resource[Any]
    dataset: Dataset | None
    def __init__(self, resource: Resource[Any], dataset: Dataset | None = None) -> None: ...
    def get_instance(self, row: dict[str, Any]) -> Model | None: ...

class ModelInstanceLoader(BaseInstanceLoader):
    def get_queryset(self) -> QuerySet[Any]: ...

class CachedInstanceLoader(ModelInstanceLoader):
    pk_field: Field
    all_instances: dict[Any, Model]



from _typeshed import Incomplete

def CredMarshalCredential(CredType, Credential: str) -> str: ...
def CredUnmarshalCredential(MarshaledCredential: str) -> tuple[Incomplete, str]: ...
def CredIsMarshaledCredential(MarshaledCredential: str) -> bool: ...
def CredEnumerate(Filter: str | None = ..., Flags: int = ...) -> tuple[dict[str, Incomplete], ...]: ...
def CredGetTargetInfo(TargetName: str, Flags: int = ...): ...
def CredGetSessionTypes(MaximumPersistCount: int = 7) -> tuple[int, ...]: ...
def CredWriteDomainCredentials(TargetInfo, Credential, Flags: int = ...) -> None: ...
def CredReadDomainCredentials(TargetInfo, Flags: int = ...) -> tuple[Incomplete, ...]: ...
def CredDelete(TargetName: str, Type, Flags: int = ...) -> None: ...
def CredWrite(Credential, Flags: int = ...) -> None: ...
def CredRead(TargetName: str, Type, Flags: int = ...): ...
def CredRename(OldTargetName: str, NewTargetName: str, Type, Flags: int = ...): ...
def CredUICmdLinePromptForCredentials(
    TargetName: str, AuthError: int = ..., UserName: str | None = ..., Password: str | None = ..., Save: int = ..., Flags=...
) -> tuple[str, str, Incomplete]: ...
def CredUIPromptForCredentials(
    TargetName: str,
    AuthError: int = ...,
    UserName: str | None = ...,
    Password: str | None = ...,
    Save: bool = ...,
    Flags: int = ...,
    UiInfo: Incomplete | None = ...,
) -> tuple[str, str, Incomplete]: ...
def CredUIConfirmCredentials(TargetName: str, Confirm) -> None: ...
def CredUIReadSSOCredW(Realm: str | None = ...) -> str: ...
def CredUIStoreSSOCredW(Realm: str, Username: str, Password: str, Persist) -> None: ...
def CredUIParseUserName(UserName: str) -> tuple[str, str]: ...

CREDUI_FLAGS_ALWAYS_SHOW_UI: int
CREDUI_FLAGS_COMPLETE_USERNAME: int
CREDUI_FLAGS_DO_NOT_PERSIST: int
CREDUI_FLAGS_EXCLUDE_CERTIFICATES: int
CREDUI_FLAGS_EXPECT_CONFIRMATION: int
CREDUI_FLAGS_GENERIC_CREDENTIALS: int
CREDUI_FLAGS_INCORRECT_PASSWORD: int
CREDUI_FLAGS_KEEP_USERNAME: int
CREDUI_FLAGS_PASSWORD_ONLY_OK: int
CREDUI_FLAGS_PERSIST: int
CREDUI_FLAGS_PROMPT_VALID: int
CREDUI_FLAGS_REQUEST_ADMINISTRATOR: int
CREDUI_FLAGS_REQUIRE_CERTIFICATE: int
CREDUI_FLAGS_REQUIRE_SMARTCARD: int
CREDUI_FLAGS_SERVER_CREDENTIAL: int
CREDUI_FLAGS_SHOW_SAVE_CHECK_BOX: int
CREDUI_FLAGS_USERNAME_TARGET_CREDENTIALS: int
CREDUI_FLAGS_VALIDATE_USERNAME: int
CREDUI_MAX_CAPTION_LENGTH: int
CREDUI_MAX_DOMAIN_TARGET_LENGTH: int
CREDUI_MAX_GENERIC_TARGET_LENGTH: int
CREDUI_MAX_MESSAGE_LENGTH: int
CREDUI_MAX_PASSWORD_LENGTH: int
CREDUI_MAX_USERNAME_LENGTH: int
CRED_ALLOW_NAME_RESOLUTION: int
CRED_CACHE_TARGET_INFORMATION: int
CRED_ENUMERATE_ALL_CREDENTIALS: int
CRED_FLAGS_OWF_CRED_BLOB: int
CRED_FLAGS_PASSWORD_FOR_CERT: int
CRED_FLAGS_PROMPT_NOW: int
CRED_FLAGS_USERNAME_TARGET: int
CRED_FLAGS_VALID_FLAGS: int
CRED_MAX_ATTRIBUTES: int
CRED_MAX_DOMAIN_TARGET_NAME_LENGTH: int
CRED_MAX_GENERIC_TARGET_NAME_LENGTH: int
CRED_MAX_STRING_LENGTH: int
CRED_MAX_USERNAME_LENGTH: int
CRED_MAX_VALUE_SIZE: int
CRED_PERSIST_ENTERPRISE: int
CRED_PERSIST_LOCAL_MACHINE: int
CRED_PERSIST_NONE: int
CRED_PERSIST_SESSION: int
CRED_PRESERVE_CREDENTIAL_BLOB: int
CRED_TI_CREATE_EXPLICIT_CRED: int
CRED_TI_DOMAIN_FORMAT_UNKNOWN: int
CRED_TI_ONLY_PASSWORD_REQUIRED: int
CRED_TI_SERVER_FORMAT_UNKNOWN: int
CRED_TI_USERNAME_TARGET: int
CRED_TI_VALID_FLAGS: int
CRED_TI_WORKGROUP_MEMBER: int
CRED_TYPE_DOMAIN_CERTIFICATE: int
CRED_TYPE_DOMAIN_EXTENDED: int
CRED_TYPE_DOMAIN_PASSWORD: int
CRED_TYPE_DOMAIN_VISIBLE_PASSWORD: int
CRED_TYPE_GENERIC: int
CRED_TYPE_GENERIC_CERTIFICATE: int
CRED_TYPE_MAXIMUM: int
CRED_TYPE_MAXIMUM_EX: int
CertCredential: int
UsernameTargetCredential: int



from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["birank"]

@_dispatchable
def birank(
    G: Graph[_Node],
    nodes: Iterable[_Node],
    *,
    alpha: float | None = None,
    beta: float | None = None,
    top_personalization: dict[str, int] | None = None,
    bottom_personalization: dict[str, int] | None = None,
    max_iter: int = 100,
    tol: float = 1.0e-6,
    weight: str | None = "weight",
) -> dict[_Node, float]: ...



from typing import Any

class ImportExportError(Exception): ...
class FieldError(ImportExportError): ...
class WidgetError(ImportExportError): ...

class ImportError(ImportExportError):
    error: Exception
    number: int | None
    row: dict[str, Any] | None
    def __init__(self, error: Exception, number: int | None = None, row: dict[str, Any] | None = None) -> None: ...



import io
from _typeshed import StrOrBytesPath, StrPath
from collections.abc import Generator, Iterable, MutableSequence
from os import PathLike
from tarfile import _Fileobj
from tempfile import _TemporaryFileWrapper

def match_tag(tag: str) -> bool: ...
def tar(
    path: PathLike[str],
    exclude: list[str] | None = None,
    dockerfile: tuple[str | None, str | None] | None = None,
    fileobj: _Fileobj | None = None,
    gzip: bool = False,
) -> _TemporaryFileWrapper[bytes] | _Fileobj: ...
def exclude_paths(root: StrPath, patterns: MutableSequence[str], dockerfile: str | None = None) -> set[str]: ...
def build_file_list(root: str) -> list[str]: ...
def create_archive(
    root: str, files: Iterable[str] | None = None, fileobj: _Fileobj | None = None, gzip: bool = False, extra_files=None
) -> _TemporaryFileWrapper[bytes] | _Fileobj: ...
def mkbuildcontext(dockerfile: io.IOBase | StrOrBytesPath) -> _TemporaryFileWrapper[bytes]: ...
def split_path(p: str) -> list[str]: ...
def normalize_slashes(p: str) -> str: ...
def walk(root: StrPath, patterns: Iterable[str], default: bool = True) -> Generator[str]: ...

class PatternMatcher:
    patterns: list[Pattern]
    def __init__(self, patterns: Iterable[str]) -> None: ...
    def matches(self, filepath: PathLike[str]) -> bool: ...
    def walk(self, root: StrPath) -> Generator[str]: ...

class Pattern:
    exclusion: bool
    dirs: list[str]
    cleaned_pattern: str
    def __init__(self, pattern_str: str) -> None: ...
    @classmethod
    def normalize(cls, p: str) -> list[str]: ...
    def match(self, filepath: str) -> bool: ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Transit(VaultApiBase):
    def create_key(
        self,
        name,
        convergent_encryption=None,
        derived=None,
        exportable=None,
        allow_plaintext_backup=None,
        key_type=None,
        mount_point="transit",
        auto_rotate_period=None,
    ): ...
    def read_key(self, name, mount_point="transit"): ...
    def list_keys(self, mount_point="transit"): ...
    def delete_key(self, name, mount_point="transit"): ...
    def update_key_configuration(
        self,
        name,
        min_decryption_version=None,
        min_encryption_version=None,
        deletion_allowed=None,
        exportable=None,
        allow_plaintext_backup=None,
        mount_point="transit",
        auto_rotate_period=None,
    ): ...
    def rotate_key(self, name, mount_point="transit"): ...
    def export_key(self, name, key_type, version=None, mount_point="transit"): ...
    def encrypt_data(
        self,
        name,
        plaintext=None,
        context=None,
        key_version=None,
        nonce=None,
        batch_input=None,
        type=None,
        convergent_encryption=None,
        mount_point: str = "transit",
        associated_data: str | None = None,
    ): ...
    def decrypt_data(
        self,
        name,
        ciphertext=None,
        context=None,
        nonce=None,
        batch_input=None,
        mount_point: str = "transit",
        associated_data: str | None = None,
    ): ...
    def rewrap_data(
        self, name, ciphertext, context=None, key_version=None, nonce=None, batch_input=None, mount_point="transit"
    ): ...
    def generate_data_key(self, name, key_type, context=None, nonce=None, bits=None, mount_point="transit"): ...
    def generate_random_bytes(self, n_bytes=None, output_format=None, mount_point="transit"): ...
    def hash_data(self, hash_input, algorithm=None, output_format=None, mount_point="transit"): ...
    def generate_hmac(self, name, hash_input, key_version=None, algorithm=None, mount_point="transit"): ...
    def sign_data(
        self,
        name,
        hash_input=None,
        key_version=None,
        hash_algorithm=None,
        context=None,
        prehashed=None,
        signature_algorithm=None,
        marshaling_algorithm=None,
        salt_length=None,
        mount_point="transit",
        batch_input=None,
    ): ...
    def verify_signed_data(
        self,
        name,
        hash_input,
        signature=None,
        hmac=None,
        hash_algorithm=None,
        context=None,
        prehashed=None,
        signature_algorithm=None,
        salt_length=None,
        marshaling_algorithm=None,
        mount_point="transit",
    ): ...
    def backup_key(self, name, mount_point="transit"): ...
    def restore_key(self, backup, name=None, force=None, mount_point="transit"): ...
    def trim_key(self, name, min_version, mount_point="transit"): ...



from typing import Final

from .classic import deprecated as deprecated
from .params import deprecated_params as deprecated_params

__version__: Final[str]
__author__: Final[str]
__date__: Final[str]
__credits__: Final[str]



from re import Pattern
from typing import Any, ClassVar

from markdown import blockparser
from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension

PIPE_NONE: int
PIPE_LEFT: int
PIPE_RIGHT: int

class TableProcessor(BlockProcessor):
    RE_CODE_PIPES: ClassVar[Pattern[str]]
    RE_END_BORDER: ClassVar[Pattern[str]]
    border: bool
    separator: str
    def __init__(self, parser: blockparser.BlockParser, config: dict[str, Any]) -> None: ...

class TableExtension(Extension):
    def __init__(self, **kwargs) -> None: ...

def makeExtension(**kwargs) -> TableExtension: ...



from types import FrameType
from typing import Any, ClassVar

from gevent import pywsgi
from gevent.pywsgi import WSGIHandler
from gevent.server import StreamServer
from gevent.socket import socket as GeventSocket
from gunicorn.http import Request
from gunicorn.workers.base_async import AsyncWorker

from .._types import _AddressType

VERSION: str

class GeventWorker(AsyncWorker):
    server_class: ClassVar[type[StreamServer] | None]
    wsgi_handler: ClassVar[type[WSGIHandler] | None]
    sockets: list[GeventSocket]

    def patch(self) -> None: ...
    def notify(self) -> None: ...
    def timeout_ctx(self) -> None: ...
    def run(self) -> None: ...
    def handle(self, listener: GeventSocket, client: GeventSocket, addr: _AddressType) -> None: ...
    def handle_request(self, listener_name: str, req: Request, sock: GeventSocket, addr: _AddressType) -> bool: ...
    def handle_quit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_usr1(self, sig: int, frame: FrameType | None) -> None: ...
    def init_process(self) -> None: ...

class GeventResponse:
    status: ClassVar[str | None]
    headers: ClassVar[dict[str, str] | None]
    sent: ClassVar[int | None]

    def __init__(self, status: str, headers: dict[str, str], clength: int | None) -> None: ...

class PyWSGIHandler(pywsgi.WSGIHandler):
    def log_request(self) -> None: ...
    def get_environ(self) -> dict[str, Any]: ...

class PyWSGIServer(pywsgi.WSGIServer): ...

class GeventPyWSGIWorker(GeventWorker):
    server_class: ClassVar[type[PyWSGIServer] | None]
    wsgi_handler: ClassVar[type[PyWSGIHandler] | None]



import json
import re
import types
from _typeshed import Incomplete
from collections.abc import Iterable
from typing import ClassVar

from pony.utils import localbase

class DBException(Exception):
    def __init__(exc, original_exc, *args) -> None: ...

class Warning(DBException): ...
class Error(DBException): ...
class InterfaceError(Error): ...
class DatabaseError(Error): ...
class DataError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...

def wrap_dbapi_exceptions(func, provider, *args, **kwargs): ...
def unexpected_args(attr, args) -> None: ...

version_re: re.Pattern[str]

def get_version_tuple(s: str): ...

class DBAPIProvider:
    paramstyle: ClassVar[str]
    quote_char: ClassVar[str]
    max_params_count: ClassVar[int]
    max_name_len: ClassVar[int]
    table_if_not_exists_syntax: ClassVar[bool]
    index_if_not_exists_syntax: ClassVar[bool]
    max_time_precision: ClassVar[int]
    default_time_precision: ClassVar[int]
    uint64_support: ClassVar[bool]
    varchar_default_max_len: ClassVar[int | None]
    dialect: ClassVar[str | None]
    dbapi_module: ClassVar[types.ModuleType | None]
    dbschema_cls: ClassVar[type | None]
    translator_cls: ClassVar[type | None]
    sqlbuilder_cls: ClassVar[type | None]
    array_converter_cls: ClassVar[type | None]
    name_before_table: ClassVar[str]
    default_schema_name: ClassVar[str | None]
    fk_types: ClassVar[dict[str, str]]
    converter_classes: Incomplete
    def __init__(provider, _database, *args, **kwargs) -> None: ...
    def inspect_connection(provider, connection) -> None: ...
    def normalize_name(provider, name): ...
    def get_default_entity_table_name(provider, entity): ...
    def get_default_m2m_table_name(provider, attr, reverse): ...
    def get_default_column_names(provider, attr, reverse_pk_columns=None): ...
    def get_default_m2m_column_names(provider, entity): ...
    def get_default_index_name(
        provider, table_name, column_names, is_pk: bool = False, is_unique: bool = False, m2m: bool = False
    ): ...
    def get_default_fk_name(provider, child_table_name, parent_table_name, child_column_names): ...
    def split_table_name(provider, table_name): ...
    def base_name(provider, name): ...
    def quote_name(provider, name: str | Iterable[str]) -> str: ...
    def format_table_name(provider, name): ...
    def normalize_vars(provider, vars, vartypes) -> None: ...
    def ast2sql(provider, ast): ...
    def should_reconnect(provider, exc): ...
    def connect(provider): ...
    def set_transaction_mode(provider, connection, cache) -> None: ...
    def commit(provider, connection, cache=None) -> None: ...
    def rollback(provider, connection, cache=None) -> None: ...
    def release(provider, connection, cache=None) -> None: ...
    def drop(provider, connection, cache=None) -> None: ...
    def disconnect(provider) -> None: ...
    def execute(provider, cursor, sql, arguments=None, returning_id: bool = False): ...
    def get_converter_by_py_type(provider, py_type): ...
    def get_converter_by_attr(provider, attr): ...
    def get_pool(provider, *args, **kwargs): ...
    def table_exists(provider, connection, table_name, case_sensitive: bool = True) -> None: ...
    def index_exists(provider, connection, table_name, index_name, case_sensitive: bool = True) -> None: ...
    def fk_exists(provider, connection, table_name, fk_name, case_sensitive: bool = True) -> None: ...
    def table_has_data(provider, connection, table_name): ...
    def disable_fk_checks(provider, connection) -> None: ...
    def enable_fk_checks(provider, connection, prev_state) -> None: ...
    def drop_table(provider, connection, table_name) -> None: ...

class Pool(localbase):
    forked_connections: list[tuple[Incomplete, int | None]]
    dbapi_module: types.ModuleType
    args: tuple[Incomplete, ...]
    kwargs: dict[str, Incomplete]
    con: Incomplete
    pid: int | None
    def __init__(pool, dbapi_module: types.ModuleType, *args, **kwargs) -> None: ...
    def connect(pool) -> tuple[Incomplete, bool]: ...
    def release(pool, con) -> None: ...
    def drop(pool, con) -> None: ...
    def disconnect(pool) -> None: ...

class Converter:
    EQ: str
    NE: str
    optimistic: bool
    def __deepcopy__(converter, memo): ...
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def init(converter, kwargs) -> None: ...
    def validate(converter, val, obj=None): ...
    def py2sql(converter, val): ...
    def sql2py(converter, val): ...
    def val2dbval(self, val, obj=None): ...
    def dbval2val(self, dbval, obj=None): ...
    def dbvals_equal(self, x, y): ...
    def get_sql_type(converter, attr=None): ...
    def get_fk_type(converter, sql_type): ...

class NoneConverter(Converter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def get_sql_type(converter, attr=None) -> None: ...
    def get_fk_type(converter, sql_type) -> None: ...

class BoolConverter(Converter):
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class StrConverter(Converter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def init(converter, kwargs) -> None: ...
    def validate(converter, val, obj=None): ...
    def sql_type(converter): ...

class IntConverter(Converter):
    signed_types: Incomplete
    unsigned_types: Incomplete
    def init(converter, kwargs) -> None: ...
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class RealConverter(Converter):
    EQ: str
    NE: str
    default_tolerance: float
    optimistic: bool
    def init(converter, kwargs) -> None: ...
    def validate(converter, val, obj=None): ...
    def dbvals_equal(converter, x, y): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class DecimalConverter(Converter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def init(converter, kwargs) -> None: ...
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class BlobConverter(Converter):
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class DateConverter(Converter):
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class ConverterWithMicroseconds(Converter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def init(converter, kwargs) -> None: ...
    def round_microseconds_to_precision(converter, microseconds, precision): ...
    def sql_type(converter): ...

class TimeConverter(ConverterWithMicroseconds):
    sql_type_name: ClassVar[str]
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...

class TimedeltaConverter(ConverterWithMicroseconds):
    sql_type_name: ClassVar[str]
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...

class DatetimeConverter(ConverterWithMicroseconds):
    sql_type_name: ClassVar[str]
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...

class UuidConverter(Converter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def validate(converter, val, obj=None): ...
    def py2sql(converter, val): ...
    sql2py = validate
    def sql_type(converter): ...

class JsonConverter(Converter):
    json_kwargs: Incomplete

    class JsonEncoder(json.JSONEncoder):
        def default(converter, obj): ...

    def validate(converter, val, obj=None): ...
    def val2dbval(converter, val, obj=None): ...
    def dbval2val(converter, dbval, obj=None): ...
    def dbvals_equal(converter, x, y): ...
    def sql_type(converter): ...

class ArrayConverter(Converter):
    array_types: Incomplete
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def validate(converter, val, obj=None): ...
    def dbval2val(converter, dbval, obj=None): ...
    def val2dbval(converter, val, obj=None): ...
    def sql_type(converter): ...



from typing import overload
from typing_extensions import deprecated

import numpy as np
from numpy.typing import NDArray

from .._typing import ArrayLike, ArrayLikeSeq
from ..lib import Geometry
from ..prepared import PreparedGeometry

@overload
@deprecated("Use 'shapely.contains_xy' instead (available since shapely 2.0.0).")
def contains(geometry: Geometry | PreparedGeometry[Geometry], x: float, y: float) -> np.bool_: ...
@overload
@deprecated("Use 'shapely.contains_xy' instead (available since shapely 2.0.0).")
def contains(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLikeSeq[float], y: ArrayLike[float]
) -> NDArray[np.bool_]: ...
@overload
@deprecated("Use 'shapely.contains_xy' instead (available since shapely 2.0.0).")
def contains(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLike[float], y: ArrayLikeSeq[float]
) -> NDArray[np.bool_]: ...
@overload
@deprecated("Use 'shapely.contains_xy' instead (available since shapely 2.0.0).")
def contains(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLike[float], y: ArrayLike[float]
) -> np.bool_ | NDArray[np.bool_]: ...
@overload
@deprecated("Use 'shapely.intersects_xy' instead (available since shapely 2.0.0).")
def touches(geometry: Geometry | PreparedGeometry[Geometry], x: float, y: float) -> np.bool_: ...
@overload
@deprecated("Use 'shapely.intersects_xy' instead (available since shapely 2.0.0).")
def touches(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLikeSeq[float], y: ArrayLike[float]
) -> NDArray[np.bool_]: ...
@overload
@deprecated("Use 'shapely.intersects_xy' instead (available since shapely 2.0.0).")
def touches(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLike[float], y: ArrayLikeSeq[float]
) -> NDArray[np.bool_]: ...
@overload
@deprecated("Use 'shapely.intersects_xy' instead (available since shapely 2.0.0).")
def touches(
    geometry: Geometry | PreparedGeometry[Geometry], x: ArrayLike[float], y: ArrayLike[float]
) -> np.bool_ | NDArray[np.bool_]: ...



from typing import Generic, TypeVar, type_check_only

from google.protobuf.descriptor import EnumDescriptor

_V = TypeVar("_V", bound=int)

# Expose a generic version so that those using mypy-protobuf
# can get autogenerated NewType wrapper around the int values
# NOTE: this doesn't actually inherit from type,
# but mypy doesn't support metaclasses that don't inherit from type,
# so we pretend it does in the stubs...
@type_check_only
class _EnumTypeWrapper(type, Generic[_V]):
    DESCRIPTOR: EnumDescriptor
    def __init__(self, enum_type: EnumDescriptor) -> None: ...
    def Name(self, number: _V) -> str: ...
    def Value(self, name: str | bytes) -> _V: ...
    def keys(self) -> list[str]: ...
    def values(self) -> list[_V]: ...
    def items(self) -> list[tuple[str, _V]]: ...

class EnumTypeWrapper(_EnumTypeWrapper[int]): ...
